#include "Data.h"

namespace ExampleData {
    bool Example::compare(points_t diff,
                          const std::vector<points_t>& reconstructs,
                          scalars_t dt, scalars_t phi, points_t cache,
                          coeff_t coeffs) const
    {
        if(diff && *diff != this->diff)
            U_EXCEPTION("diff doesnt match example.");
        
        if(dt && !float_equals(*this->dt, *dt, 1e-6))
            U_EXCEPTION("dt doesnt match example data.");
        //if(!float_equals(*cumsum, *expected_cumsum, 1e-8))
        //    U_EXCEPTION("cumsum doesnt match example.");
        
        if(phi && !float_equals(*this->phi, *phi, 1e-6))
            U_EXCEPTION("phi doesnt match example data.");
        
        if(cache && !float_equals(*this->cache, *cache, 1e-6))
            U_EXCEPTION("cache doesnt match example data.");
        
        if(coeffs) {
            std::vector<scalar_t> expected_flat((scalar_t*)this->coeffs->data(), (scalar_t*)this->coeffs->data() + this->coeffs->size() * 4);
            std::vector<scalar_t> flat((scalar_t*)coeffs->data(), (scalar_t*)coeffs->data() + coeffs->size() * 4);
            
            if(!float_equals(expected_flat, flat, 1e-4))
                U_EXCEPTION("coeffs didnt match example data.");
        }
        
        if(!reconstructs.empty()) {
            if (reconstructs.size() != this->reconstruction.size()) {
                U_EXCEPTION("Mismatch of number of reconstructions: %d != %d", reconstructs.size(), this->reconstruction.size());
            }
            for (size_t i=0; i<reconstructs.size(); ++i) {
                if(!float_equals(this->reconstruction.at(i), *reconstructs.at(i), 1e-4))
                    U_EXCEPTION("reconstructed data at order %d is not as expected.", i);
            }
        }
        
        return true;
    }
    
    Example fish() {
        coeff_t expected_coeffs = std::make_shared<coeff_t::element_type>();
        *expected_coeffs = {
            {-2.28829045e+01,  4.66541866e-01,  1.33990899e+00,
                6.80427655e+00},
            { 1.32456356e-01,  1.39484593e-01,  1.23741370e+00,
                1.71578065e+00},
            {-2.29447890e+00,  1.16876177e-01, -4.99127523e-01,
                1.05611735e+00},
            { 1.87149264e-01, -1.04975615e-01,  4.41487176e-01,
                7.05498771e-02},
            {-6.32936768e-01,  1.05350151e-01, -1.21858000e-01,
                6.60010524e-01},
            { 1.00679903e-01, -4.82293049e-02,  2.68676677e-03,
                3.42843153e-02},
            {-2.65910156e-01,  4.89879225e-02, -1.17459848e-01,
                1.82848325e-01},
            { 1.07353102e-01, -6.12311345e-02,  4.08428018e-02,
                -5.52648083e-02},
            {-1.23616084e-01,  5.69087931e-02,  1.41402345e-02,
                9.65832534e-02},
            { 6.46375358e-02, -4.36979021e-02, -2.61430531e-02,
                -7.13505811e-02}
        };
        
        return Example{
            10,
            get_fish(),
            {},
            reconstructed_fish(),
            nullptr,
            nullptr,
            nullptr,
            expected_coeffs
        };
    }

    Example ranges() {
        auto expected_dt = std::make_shared<scalars_t::element_type>();
        *expected_dt = {
            1.41421356,  1.41421356,  1.41421356,  1.41421356,  1.41421356,
            1.41421356,  1.41421356,  1.41421356,  1.41421356
        };
        
        //ASSERT(float_equals(*dt, *expected_dt, 1e-8));
        
        auto expected_cumsum = std::make_shared<scalars_t::element_type>();
        *expected_cumsum = {
            0.        ,  1.41421356,  2.82842712,  4.24264069,  5.65685425,
            7.07106781,  8.48528137,  9.89949494, 11.3137085 , 12.72792206
        };
        
        //ASSERT(float_equals(*cumsum, *expected_cumsum, 1e-8));
        
       // auto T = cumsum->back();
        
        auto expected_phi = std::make_shared<scalars_t::element_type>();
        *expected_phi = {
            0.       , 0.6981317, 1.3962634, 2.0943951, 2.7925268, 3.4906585,
            4.1887902, 4.8869219, 5.5850536, 6.2831853
        };
        
        //for(auto &ptr : *phi) {
        //    ptr /= T;
        //}
        
        //ASSERT(float_equals(*phi, *expected_phi, 1e-8));
        
        auto expected_cache = std::make_shared<points_t::element_type>();
        *expected_cache = {
            point_t( 0.70710678,  0.70710678),
            point_t( 0.70710678,  0.70710678),
            point_t( 0.70710678,  0.70710678),
            point_t( 0.70710678,  0.70710678),
            point_t( 0.70710678,  0.70710678),
            point_t( 0.70710678,  0.70710678),
            point_t( 0.70710678,  0.70710678),
            point_t( 0.70710678,  0.70710678),
            point_t( 0.70710678,  0.70710678)
        };
        
        //ASSERT(float_equals(*cache_xy_sum, *expected_cache, 1e-8));
        
        auto expected_coeffs = std::make_shared<coeff_t::element_type>();
        *expected_coeffs = {
            { 8.94845398e-17, -1.43175264e-16,  8.94845398e-17,
                -1.43175264e-16},
            {-1.78969080e-17, -7.15876319e-17, -1.78969080e-17,
                -7.15876319e-17},
            { 0.00000000e+00, -2.38625440e-17,  0.00000000e+00,
                -2.38625440e-17},
            {-8.94845398e-18, -3.46752592e-17, -8.94845398e-18,
                -3.46752592e-17},
            { 0.00000000e+00, -2.29080422e-17,  0.00000000e+00,
                -2.29080422e-17},
            {-3.97709066e-18, -1.98854533e-17, -3.97709066e-18,
                -1.98854533e-17},
            { 1.46097208e-18, -1.46097208e-17,  1.46097208e-18,
                -1.46097208e-17},
            { 1.39819593e-18, -1.51005161e-17,  1.39819593e-18,
                -1.51005161e-17},
            { 0.00000000e+00, -1.24082575e-17,  0.00000000e+00,
                -1.24082575e-17},
            { 2.50556711e-18, -1.07381448e-17,  2.50556711e-18,
                -1.07381448e-17}
        };
        
        return Example{
            10, // order
            { // points
                point_t( 0, 10),
                point_t( 1, 11),
                point_t( 2, 12),
                point_t( 3, 13),
                point_t( 4, 14),
                point_t( 5, 15),
                point_t( 6, 16),
                point_t( 7, 17),
                point_t( 8, 18),
                point_t( 9, 19)
            },
            { // diff
                point_t(1, 1),
                point_t(1, 1),
                point_t(1, 1),
                point_t(1, 1),
                point_t(1, 1),
                point_t(1, 1),
                point_t(1, 1),
                point_t(1, 1),
                point_t(1, 1),
                point_t(-9, -9)
            },
            
            {},
            
            expected_dt,
            expected_phi,
            expected_cache,
            expected_coeffs
        };
    }

    std::vector<point_t> get_fish() {
        std::vector<point_t> expected_points = {point_t(2.3911123275756836, 12.463739395141602),
            point_t(2.4911123275756837, 13.063739395141601),
            point_t(2.691112327575684, 13.7637393951416),
            point_t(2.9911123275756837, 14.3637393951416),
            point_t(3.3911123275756836, 15.0637393951416),
            point_t(3.7911123275756835, 15.5637393951416),
            point_t(4.291112327575684, 16.1637393951416),
            point_t(4.891112327575684, 16.6637393951416),
            point_t(5.491112327575683, 17.0637393951416),
            point_t(6.091112327575683, 17.3637393951416),
            point_t(6.6911123275756825, 17.6637393951416),
            point_t(7.391112327575683, 17.8637393951416),
            point_t(8.191112327575683, 17.9637393951416),
            point_t(8.891112327575682, 18.063739395141603),
            point_t(9.591112327575681, 18.063739395141603),
            point_t(10.391112327575682, 18.063739395141603),
            point_t(11.191112327575683, 17.9637393951416),
            point_t(11.891112327575682, 17.8637393951416),
            point_t(12.691112327575683, 17.7637393951416),
            point_t(13.491112327575683, 17.663739395141597),
            point_t(14.291112327575684, 17.563739395141596),
            point_t(14.991112327575683, 17.563739395141596),
            point_t(15.791112327575684, 17.463739395141594),
            point_t(16.491112327575685, 17.363739395141593),
            point_t(17.291112327575686, 17.26373939514159),
            point_t(17.991112327575685, 17.26373939514159),
            point_t(18.791112327575686, 17.16373939514159),
            point_t(19.491112327575685, 17.06373939514159),
            point_t(20.191112327575684, 16.963739395141587),
            point_t(20.991112327575685, 16.863739395141586),
            point_t(21.691112327575684, 16.863739395141586),
            point_t(22.391112327575684, 16.763739395141585),
            point_t(23.191112327575684, 16.663739395141583),
            point_t(23.891112327575684, 16.663739395141583),
            point_t(24.591112327575683, 16.56373939514158),
            point_t(25.291112327575682, 16.46373939514158),
            point_t(25.99111232757568, 16.36373939514158),
            point_t(26.69111232757568, 16.16373939514158),
            point_t(27.39111232757568, 15.96373939514158),
            point_t(28.09111232757568, 15.763739395141581),
            point_t(28.69111232757568, 15.46373939514158),
            point_t(29.39111232757568, 15.16373939514158),
            point_t(30.09111232757568, 14.863739395141579),
            point_t(30.79111232757568, 14.463739395141578),
            point_t(31.491112327575678, 14.163739395141578),
            point_t(32.19111232757568, 13.863739395141577),
            point_t(32.891112327575684, 13.563739395141576),
            point_t(33.591112327575686, 13.263739395141576),
            point_t(34.29111232757569, 12.963739395141575),
            point_t(34.99111232757569, 12.763739395141576),
            point_t(35.691112327575695, 12.563739395141576),
            point_t(36.3911123275757, 12.363739395141577),
            point_t(37.0911123275757, 12.163739395141578),
            point_t(37.7911123275757, 11.963739395141578),
            point_t(38.491112327575706, 11.863739395141579),
            point_t(39.19111232757571, 11.76373939514158),
            point_t(39.991112327575706, 11.76373939514158),
            point_t(40.69111232757571, 11.66373939514158),
            point_t(41.491112327575706, 11.66373939514158),
            point_t(42.2911123275757, 11.56373939514158),
            point_t(42.991112327575706, 11.56373939514158),
            point_t(43.7911123275757, 11.56373939514158),
            point_t(44.5911123275757, 11.56373939514158),
            point_t(45.2911123275757, 11.56373939514158),
            point_t(46.0911123275757, 11.66373939514158),
            point_t(46.7911123275757, 11.76373939514158),
            point_t(47.5911123275757, 11.863739395141579),
            point_t(48.2911123275757, 11.963739395141578),
            point_t(49.0911123275757, 12.063739395141578),
            point_t(49.7911123275757, 12.263739395141577),
            point_t(50.5911123275757, 12.363739395141577),
            point_t(51.2911123275757, 12.463739395141577),
            point_t(51.991112327575706, 12.563739395141576),
            point_t(52.69111232757571, 12.663739395141576),
            point_t(53.39111232757571, 12.663739395141576),
            point_t(53.99111232757571, 12.663739395141576),
            point_t(54.49111232757571, 12.663739395141576),
            point_t(54.99111232757571, 12.463739395141577),
            point_t(55.29111232757571, 12.263739395141577),
            point_t(55.39111232757571, 11.963739395141577),
            point_t(55.39111232757571, 11.563739395141576),
            point_t(55.39111232757571, 11.163739395141576),
            point_t(55.19111232757571, 10.663739395141576),
            point_t(54.79111232757571, 10.163739395141576),
            point_t(54.39111232757571, 9.763739395141576),
            point_t(53.79111232757571, 9.263739395141576),
            point_t(53.19111232757571, 8.863739395141575),
            point_t(52.59111232757571, 8.463739395141575),
            point_t(51.991112327575706, 8.063739395141575),
            point_t(51.2911123275757, 7.663739395141574),
            point_t(50.5911123275757, 7.263739395141574),
            point_t(49.8911123275757, 6.963739395141574),
            point_t(49.191112327575695, 6.663739395141574),
            point_t(48.59111232757569, 6.363739395141574),
            point_t(47.791112327575696, 6.063739395141575),
            point_t(47.09111232757569, 5.763739395141575),
            point_t(46.39111232757569, 5.563739395141575),
            point_t(45.69111232757569, 5.363739395141574),
            point_t(44.991112327575685, 5.163739395141574),
            point_t(44.29111232757568, 4.963739395141574),
            point_t(43.491112327575685, 4.863739395141574),
            point_t(42.69111232757569, 4.663739395141574),
            point_t(41.991112327575685, 4.563739395141575),
            point_t(41.19111232757569, 4.463739395141575),
            point_t(40.491112327575685, 4.263739395141575),
            point_t(39.69111232757569, 4.163739395141575),
            point_t(38.89111232757569, 4.0637393951415755),
            point_t(38.19111232757569, 3.8637393951415753),
            point_t(37.39111232757569, 3.763739395141575),
            point_t(36.69111232757569, 3.563739395141575),
            point_t(35.89111232757569, 3.463739395141575),
            point_t(35.19111232757569, 3.363739395141575),
            point_t(34.39111232757569, 3.1637393951415747),
            point_t(33.69111232757569, 3.0637393951415746),
            point_t(32.89111232757569, 2.9637393951415745),
            point_t(32.19111232757569, 2.8637393951415744),
            point_t(31.391112327575687, 2.7637393951415743),
            point_t(30.591112327575686, 2.663739395141574),
            point_t(29.791112327575686, 2.663739395141574),
            point_t(28.991112327575685, 2.563739395141574),
            point_t(28.191112327575684, 2.563739395141574),
            point_t(27.391112327575684, 2.563739395141574),
            point_t(26.591112327575683, 2.563739395141574),
            point_t(25.791112327575682, 2.563739395141574),
            point_t(24.99111232757568, 2.563739395141574),
            point_t(24.19111232757568, 2.563739395141574),
            point_t(23.39111232757568, 2.563739395141574),
            point_t(22.59111232757568, 2.563739395141574),
            point_t(21.79111232757568, 2.563739395141574),
            point_t(20.991112327575678, 2.563739395141574),
            point_t(20.29111232757568, 2.563739395141574),
            point_t(19.491112327575678, 2.663739395141574),
            point_t(18.691112327575677, 2.7637393951415743),
            point_t(17.891112327575676, 2.8637393951415744),
            point_t(17.091112327575676, 2.9637393951415745),
            point_t(16.291112327575675, 3.1637393951415747),
            point_t(15.491112327575674, 3.2637393951415747),
            point_t(14.791112327575675, 3.463739395141575),
            point_t(13.991112327575674, 3.563739395141575),
            point_t(13.291112327575675, 3.763739395141575),
            point_t(12.491112327575674, 3.8637393951415753),
            point_t(11.791112327575675, 4.0637393951415755),
            point_t(10.991112327575674, 4.263739395141576),
            point_t(10.291112327575675, 4.463739395141576),
            point_t(9.591112327575676, 4.663739395141576),
            point_t(8.891112327575676, 4.863739395141576),
            point_t(8.191112327575677, 5.063739395141576),
            point_t(7.491112327575677, 5.363739395141576),
            point_t(6.791112327575677, 5.663739395141576),
            point_t(6.091112327575677, 5.963739395141576),
            point_t(5.491112327575677, 6.363739395141576),
            point_t(4.891112327575677, 6.863739395141576),
            point_t(4.391112327575677, 7.363739395141576),
            point_t(3.8911123275756774, 7.863739395141576),
            point_t(3.3911123275756774, 8.463739395141577),
            point_t(3.0911123275756776, 9.063739395141576),
            point_t(2.7911123275756777, 9.663739395141576),
            point_t(2.5911123275756776, 10.363739395141575),
            point_t(2.3911123275756774, 11.063739395141575),
            point_t(2.3911123275756774, 11.763739395141574)};
        return expected_points;
    }

    std::vector<point_t> get_termite() {
        return {point_t(135.1, 58.1), point_t(136.0, 57.5), point_t(136.5, 57.5), point_t(137.2, 56.8), point_t(138.5, 56.5), point_t(139.5, 56.5), point_t(140.2, 55.8), point_t(141.0, 55.5), point_t(141.5, 55.5), point_t(142.2, 54.8), point_t(144.0, 54.5), point_t(144.5, 54.5), point_t(145.2, 53.8), point_t(146.5, 53.5), point_t(147.5, 53.5), point_t(148.5, 53.5), point_t(149.2, 52.8), point_t(150.0, 52.5), point_t(150.8, 51.9), point_t(151.5, 51.5), point_t(152.2, 50.8), point_t(153.5, 50.5), point_t(154.5, 50.5), point_t(155.2, 49.8), point_t(156.0, 49.5), point_t(156.5, 49.5), point_t(157.2, 48.8), point_t(158.5, 48.5), point_t(158.8, 47.9), point_t(159.8, 46.9), point_t(160.5, 46.5), point_t(160.5, 45.5), point_t(160.5, 44.5), point_t(160.5, 43.5), point_t(160.5, 42.5), point_t(160.5, 41.5), point_t(159.8, 40.8), point_t(159.60000000000002, 39.8), point_t(159.50000000000003, 39.5), point_t(159.50000000000003, 38.5), point_t(159.50000000000003, 37.5), point_t(159.50000000000003, 36.0), point_t(159.50000000000003, 35.5), point_t(158.80000000000004, 34.8), point_t(158.80000000000004, 33.3), point_t(159.50000000000003, 32.5), point_t(159.50000000000003, 31.5), point_t(158.80000000000004, 30.8), point_t(158.80000000000004, 29.8), point_t(158.80000000000004, 29.3), point_t(159.50000000000003, 28.0), point_t(159.50000000000003, 27.5), point_t(159.50000000000003, 26.0), point_t(159.50000000000003, 25.5), point_t(158.90000000000003, 24.5), point_t(159.50000000000003, 23.5), point_t(159.50000000000003, 22.5), point_t(159.70000000000002, 21.3), point_t(160.50000000000003, 20.5), point_t(160.50000000000003, 20.0), point_t(161.10000000000002, 19.1), point_t(162.00000000000003, 18.5), point_t(162.90000000000003, 18.5), point_t(163.60000000000002, 18.4), point_t(164.3, 16.9), point_t(165.20000000000002, 16.599999999999998), point_t(165.50000000000003, 16.499999999999996), point_t(165.80000000000004, 15.399999999999997), point_t(164.90000000000003, 15.199999999999998), point_t(164.10000000000002, 15.099999999999998), point_t(164.00000000000003, 15.999999999999998), point_t(163.90000000000003, 17.0), point_t(163.80000000000004, 18.0), point_t(163.70000000000005, 19.0), point_t(163.50000000000006, 20.5), point_t(163.50000000000006, 21.5), point_t(163.50000000000006, 22.5), point_t(163.50000000000006, 23.5), point_t(163.50000000000006, 24.5), point_t(163.50000000000006, 25.5), point_t(163.50000000000006, 26.5), point_t(163.50000000000006, 27.5), point_t(164.20000000000005, 28.2), point_t(164.50000000000006, 29.5), point_t(164.50000000000006, 30.5), point_t(164.50000000000006, 31.5), point_t(164.50000000000006, 32.5), point_t(165.20000000000005, 33.2), point_t(165.20000000000005, 34.2), point_t(165.20000000000005, 34.7), point_t(164.50000000000006, 35.5), point_t(165.20000000000005, 36.2), point_t(165.50000000000006, 37.5), point_t(165.50000000000006, 38.5), point_t(165.50000000000006, 39.5), point_t(165.50000000000006, 40.5), point_t(165.50000000000006, 41.5), point_t(165.50000000000006, 42.5), point_t(166.20000000000005, 43.2), point_t(166.40000000000003, 44.2), point_t(166.50000000000003, 44.5), point_t(166.50000000000003, 45.5), point_t(166.30000000000004, 46.7), point_t(165.50000000000003, 47.5), point_t(166.10000000000002, 48.5), point_t(165.50000000000003, 49.5), point_t(165.50000000000003, 50.0), point_t(165.30000000000004, 50.7), point_t(164.20000000000005, 52.1), point_t(163.50000000000006, 52.5), point_t(163.50000000000006, 53.0), point_t(162.90000000000006, 53.9), point_t(161.50000000000006, 54.5), point_t(160.80000000000007, 55.2), point_t(159.80000000000007, 55.400000000000006), point_t(159.00000000000006, 55.50000000000001), point_t(158.50000000000006, 55.50000000000001), point_t(157.80000000000007, 56.20000000000001), point_t(156.00000000000006, 56.50000000000001), point_t(155.50000000000006, 56.50000000000001), point_t(154.80000000000007, 57.20000000000001), point_t(153.50000000000006, 57.50000000000001), point_t(152.50000000000006, 57.50000000000001), point_t(151.80000000000007, 58.20000000000001), point_t(151.00000000000006, 58.50000000000001), point_t(150.50000000000006, 58.50000000000001), point_t(149.80000000000007, 59.20000000000001), point_t(148.50000000000006, 59.50000000000001), point_t(147.50000000000006, 59.50000000000001), point_t(146.80000000000007, 60.20000000000001), point_t(145.50000000000006, 60.50000000000001), point_t(144.80000000000007, 61.20000000000001), point_t(144.00000000000006, 61.50000000000001), point_t(143.50000000000006, 62.00000000000001), point_t(142.90000000000006, 62.900000000000006), point_t(142.00000000000006, 63.2), point_t(141.00000000000006, 63.5), point_t(140.50000000000006, 64.0), point_t(139.90000000000006, 64.9), point_t(139.00000000000006, 65.2), point_t(138.00000000000006, 65.5), point_t(137.50000000000006, 65.5), point_t(136.80000000000007, 66.2), point_t(135.50000000000006, 66.5), point_t(135.50000000000006, 67.5), point_t(136.50000000000006, 67.5), point_t(137.50000000000006, 67.5), point_t(138.50000000000006, 67.5), point_t(139.50000000000006, 67.5), point_t(140.50000000000006, 67.5), point_t(141.50000000000006, 67.5), point_t(142.50000000000006, 67.5), point_t(143.60000000000005, 67.2), point_t(144.30000000000004, 67.8), point_t(144.50000000000003, 68.5), point_t(145.10000000000002, 68.8), point_t(146.00000000000003, 69.5), point_t(146.90000000000003, 70.1), point_t(147.20000000000005, 71.0), point_t(147.50000000000006, 72.0), point_t(147.50000000000006, 72.5), point_t(148.20000000000005, 73.2), point_t(148.50000000000006, 74.5), point_t(148.80000000000007, 73.6), point_t(149.10000000000008, 73.1), point_t(150.50000000000009, 72.5), point_t(151.50000000000009, 72.5), point_t(152.50000000000009, 72.5), point_t(153.70000000000007, 72.7), point_t(154.50000000000009, 73.5), point_t(155.50000000000009, 73.5), point_t(156.50000000000009, 73.5), point_t(157.50000000000009, 73.5), point_t(158.20000000000007, 72.8), point_t(158.20000000000007, 72.8), point_t(159.70000000000007, 72.8), point_t(160.50000000000009, 73.5), point_t(161.50000000000009, 73.5), point_t(162.70000000000007, 73.7), point_t(163.50000000000009, 74.5), point_t(164.50000000000009, 74.5), point_t(165.4000000000001, 74.6), point_t(165.7000000000001, 74.69999999999999), point_t(166.5000000000001, 75.49999999999999), point_t(167.5000000000001, 76.49999999999999), point_t(168.1000000000001, 76.79999999999998), point_t(169.0000000000001, 77.49999999999999), point_t(169.5000000000001, 77.49999999999999), point_t(170.5000000000001, 77.49999999999999), point_t(171.5000000000001, 77.49999999999999), point_t(172.5000000000001, 77.49999999999999), point_t(173.5000000000001, 77.49999999999999), point_t(174.5000000000001, 77.49999999999999), point_t(175.5000000000001, 77.49999999999999), point_t(176.5000000000001, 77.49999999999999), point_t(177.7000000000001, 77.69999999999999), point_t(178.5000000000001, 78.49999999999999), point_t(179.5000000000001, 78.49999999999999), point_t(180.5000000000001, 78.49999999999999), point_t(181.5000000000001, 78.49999999999999), point_t(182.5000000000001, 78.49999999999999), point_t(183.5000000000001, 78.49999999999999), point_t(184.5000000000001, 78.49999999999999), point_t(185.5000000000001, 78.49999999999999), point_t(186.5000000000001, 78.49999999999999), point_t(187.5000000000001, 78.49999999999999), point_t(188.5000000000001, 78.49999999999999), point_t(189.5000000000001, 78.49999999999999), point_t(190.5000000000001, 78.49999999999999), point_t(191.5000000000001, 78.49999999999999), point_t(192.5000000000001, 78.49999999999999), point_t(193.5000000000001, 78.49999999999999), point_t(194.5000000000001, 78.49999999999999), point_t(195.5000000000001, 78.49999999999999), point_t(196.5000000000001, 78.49999999999999), point_t(197.5000000000001, 78.49999999999999), point_t(198.5000000000001, 78.49999999999999), point_t(199.5000000000001, 78.49999999999999), point_t(200.5000000000001, 78.49999999999999), point_t(201.5000000000001, 78.49999999999999), point_t(202.5000000000001, 78.49999999999999), point_t(203.5000000000001, 78.49999999999999), point_t(204.5000000000001, 78.49999999999999), point_t(205.5000000000001, 78.49999999999999), point_t(206.5000000000001, 78.49999999999999), point_t(207.5000000000001, 78.49999999999999), point_t(208.2000000000001, 77.79999999999998), point_t(209.5000000000001, 77.49999999999999), point_t(210.5000000000001, 77.49999999999999), point_t(211.5000000000001, 77.49999999999999), point_t(212.5000000000001, 77.49999999999999), point_t(213.5000000000001, 77.49999999999999), point_t(214.5000000000001, 77.49999999999999), point_t(215.5000000000001, 77.49999999999999), point_t(216.5000000000001, 77.49999999999999), point_t(217.5000000000001, 77.49999999999999), point_t(218.5000000000001, 77.49999999999999), point_t(219.2000000000001, 76.79999999999998), point_t(220.0000000000001, 76.49999999999999), point_t(220.5000000000001, 76.49999999999999), point_t(221.2000000000001, 75.79999999999998), point_t(222.7000000000001, 75.79999999999998), point_t(223.5000000000001, 76.49999999999999), point_t(224.5000000000001, 76.49999999999999), point_t(225.5000000000001, 76.49999999999999), point_t(226.5000000000001, 76.49999999999999), point_t(227.2000000000001, 75.79999999999998), point_t(228.2000000000001, 75.59999999999998), point_t(229.0000000000001, 75.49999999999999), point_t(229.5000000000001, 75.49999999999999), point_t(230.2000000000001, 74.79999999999998), point_t(231.7000000000001, 74.79999999999998), point_t(232.5000000000001, 75.49999999999999), point_t(232.7000000000001, 76.39999999999999), point_t(232.8000000000001, 76.6), point_t(231.50000000000009, 77.5), point_t(230.8000000000001, 78.2), point_t(229.8000000000001, 78.4), point_t(229.50000000000009, 78.5), point_t(228.50000000000009, 78.5), point_t(227.50000000000009, 78.5), point_t(226.50000000000009, 78.5), point_t(225.50000000000009, 78.5), point_t(224.00000000000009, 78.5), point_t(223.50000000000009, 78.5), point_t(222.8000000000001, 79.2), point_t(221.00000000000009, 79.5), point_t(220.50000000000009, 79.5), point_t(219.8000000000001, 80.2), point_t(218.50000000000009, 80.5), point_t(217.50000000000009, 80.5), point_t(216.50000000000009, 80.5), point_t(215.50000000000009, 80.5), point_t(214.8000000000001, 81.2), point_t(213.8000000000001, 81.4), point_t(213.50000000000009, 81.5), point_t(212.00000000000009, 81.5), point_t(211.50000000000009, 82.0), point_t(212.00000000000009, 82.5), point_t(212.50000000000009, 82.5), point_t(213.9000000000001, 83.1), point_t(213.9000000000001, 84.1), point_t(213.9000000000001, 84.89999999999999), point_t(212.50000000000009, 85.49999999999999), point_t(211.50000000000009, 85.49999999999999), point_t(210.50000000000009, 85.49999999999999), point_t(209.50000000000009, 85.49999999999999), point_t(208.50000000000009, 85.49999999999999), point_t(207.50000000000009, 85.49999999999999), point_t(206.50000000000009, 85.49999999999999), point_t(205.50000000000009, 85.49999999999999), point_t(204.50000000000009, 85.49999999999999), point_t(203.60000000000008, 85.39999999999999), point_t(203.30000000000007, 85.3), point_t(202.00000000000006, 84.5), point_t(201.30000000000007, 84.3), point_t(200.50000000000006, 83.5), point_t(199.50000000000006, 83.5), point_t(198.50000000000006, 83.5), point_t(197.50000000000006, 83.5), point_t(196.80000000000007, 84.2), point_t(195.80000000000007, 84.2), point_t(195.30000000000007, 84.2), point_t(194.00000000000006, 83.5), point_t(193.50000000000006, 83.5), point_t(192.00000000000006, 83.5), point_t(191.50000000000006, 83.5), point_t(190.80000000000007, 84.2), point_t(189.50000000000006, 84.5), point_t(188.80000000000007, 85.2), point_t(187.50000000000006, 85.5), point_t(188.00000000000006, 86.3), point_t(188.10000000000005, 86.5), point_t(187.50000000000006, 88.0), point_t(187.50000000000006, 88.5), point_t(188.20000000000005, 89.2), point_t(188.50000000000006, 90.5), point_t(188.50000000000006, 91.5), point_t(189.20000000000005, 92.2), point_t(189.50000000000006, 93.5), point_t(189.50000000000006, 94.5), point_t(190.50000000000006, 94.5), point_t(191.50000000000006, 94.5), point_t(192.40000000000006, 94.6), point_t(192.70000000000007, 94.69999999999999), point_t(193.50000000000009, 95.49999999999999), point_t(194.50000000000009, 96.49999999999999), point_t(195.50000000000009, 96.49999999999999), point_t(196.4000000000001, 96.59999999999998), point_t(196.7000000000001, 96.69999999999997), point_t(198.1000000000001, 97.79999999999997), point_t(198.5000000000001, 98.49999999999997), point_t(199.5000000000001, 98.49999999999997), point_t(200.7000000000001, 98.69999999999997), point_t(201.5000000000001, 99.49999999999997), point_t(202.5000000000001, 99.49999999999997), point_t(203.40000000000012, 99.59999999999997), point_t(203.70000000000013, 99.69999999999996), point_t(204.50000000000014, 100.49999999999996), point_t(205.50000000000014, 101.49999999999996), point_t(206.00000000000014, 101.49999999999996), point_t(206.70000000000013, 101.69999999999996), point_t(207.50000000000014, 102.49999999999996), point_t(207.80000000000015, 103.59999999999995), point_t(206.50000000000014, 104.49999999999996), point_t(205.50000000000014, 104.49999999999996), point_t(204.50000000000014, 104.49999999999996), point_t(203.50000000000014, 104.49999999999996), point_t(202.50000000000014, 104.49999999999996), point_t(201.50000000000014, 104.49999999999996), point_t(200.50000000000014, 104.49999999999996), point_t(199.50000000000014, 104.49999999999996), point_t(198.50000000000014, 104.49999999999996), point_t(197.50000000000014, 104.49999999999996), point_t(196.50000000000014, 104.49999999999996), point_t(195.50000000000014, 104.49999999999996), point_t(194.50000000000014, 104.49999999999996), point_t(193.50000000000014, 104.49999999999996), point_t(192.50000000000014, 104.49999999999996), point_t(191.50000000000014, 104.49999999999996), point_t(190.50000000000014, 104.49999999999996), point_t(191.20000000000013, 105.19999999999996), point_t(191.50000000000014, 105.99999999999996), point_t(191.50000000000014, 106.49999999999996), point_t(192.50000000000014, 107.49999999999996), point_t(193.50000000000014, 107.49999999999996), point_t(194.50000000000014, 107.49999999999996), point_t(195.20000000000013, 106.79999999999995), point_t(196.20000000000013, 106.59999999999995), point_t(196.50000000000014, 106.49999999999996), point_t(197.50000000000014, 106.49999999999996), point_t(198.50000000000014, 106.49999999999996), point_t(199.50000000000014, 106.49999999999996), point_t(200.70000000000013, 106.69999999999996), point_t(202.00000000000014, 107.49999999999996), point_t(203.00000000000014, 107.49999999999996), point_t(203.50000000000014, 107.49999999999996), point_t(204.20000000000013, 106.79999999999995), point_t(205.50000000000014, 106.49999999999996), point_t(206.40000000000015, 106.59999999999995), point_t(206.70000000000016, 106.69999999999995), point_t(208.00000000000017, 107.49999999999994), point_t(208.90000000000018, 108.09999999999994), point_t(208.90000000000018, 109.09999999999994), point_t(208.90000000000018, 109.89999999999993), point_t(207.50000000000017, 110.49999999999993), point_t(206.50000000000017, 110.49999999999993), point_t(205.50000000000017, 110.49999999999993), point_t(204.50000000000017, 110.49999999999993), point_t(203.50000000000017, 110.49999999999993), point_t(202.60000000000016, 110.39999999999993), point_t(202.30000000000015, 110.29999999999994), point_t(201.00000000000014, 109.49999999999994), point_t(200.50000000000014, 109.49999999999994), point_t(199.50000000000014, 109.49999999999994), point_t(198.00000000000014, 109.49999999999994), point_t(197.50000000000014, 109.49999999999994), point_t(196.80000000000015, 110.19999999999995), point_t(195.30000000000015, 110.19999999999995), point_t(194.50000000000014, 109.49999999999994), point_t(193.50000000000014, 109.49999999999994), point_t(192.50000000000014, 109.49999999999994), point_t(191.50000000000014, 109.49999999999994), point_t(190.50000000000014, 109.49999999999994), point_t(189.50000000000014, 109.49999999999994), point_t(188.60000000000014, 109.39999999999995), point_t(188.30000000000013, 109.29999999999995), point_t(187.5000000000001, 108.49999999999996), point_t(186.5000000000001, 107.49999999999996), point_t(186.0000000000001, 107.49999999999996), point_t(185.1000000000001, 106.89999999999996), point_t(184.5000000000001, 105.99999999999996), point_t(183.5000000000001, 105.49999999999996), point_t(182.5000000000001, 105.49999999999996), point_t(181.5000000000001, 105.49999999999996), point_t(181.30000000000013, 106.69999999999996), point_t(180.60000000000014, 107.39999999999996), point_t(180.20000000000013, 108.09999999999997), point_t(179.50000000000014, 108.99999999999997), point_t(179.50000000000014, 109.49999999999997), point_t(180.20000000000013, 110.19999999999997), point_t(180.50000000000014, 111.49999999999997), point_t(180.50000000000014, 112.49999999999997), point_t(181.20000000000013, 113.19999999999997), point_t(181.40000000000012, 114.19999999999997), point_t(181.5000000000001, 114.49999999999997), point_t(181.5000000000001, 115.49999999999997), point_t(181.5000000000001, 116.49999999999997), point_t(181.5000000000001, 117.49999999999997), point_t(181.5000000000001, 118.49999999999997), point_t(181.5000000000001, 119.49999999999997), point_t(181.5000000000001, 120.49999999999997), point_t(181.5000000000001, 121.99999999999997), point_t(181.5000000000001, 122.49999999999997), point_t(182.2000000000001, 123.19999999999997), point_t(182.5000000000001, 124.49999999999997), point_t(182.5000000000001, 125.99999999999997), point_t(183.0000000000001, 126.49999999999997), point_t(183.5000000000001, 126.49999999999997), point_t(184.7000000000001, 126.69999999999997), point_t(185.5000000000001, 127.49999999999997), point_t(186.0000000000001, 127.49999999999997), point_t(186.7000000000001, 127.69999999999997), point_t(188.1000000000001, 128.79999999999998), point_t(188.5000000000001, 129.49999999999997), point_t(189.1000000000001, 129.79999999999998), point_t(190.1000000000001, 130.79999999999998), point_t(190.5000000000001, 131.49999999999997), point_t(191.0000000000001, 131.49999999999997), point_t(191.90000000000012, 132.09999999999997), point_t(192.5000000000001, 132.99999999999997), point_t(192.6000000000001, 134.59999999999997), point_t(191.7000000000001, 134.59999999999997), point_t(191.0000000000001, 134.49999999999997), point_t(190.5000000000001, 134.49999999999997), point_t(189.30000000000013, 134.29999999999998), point_t(188.5000000000001, 133.49999999999997), point_t(187.5000000000001, 132.49999999999997), point_t(187.0000000000001, 132.49999999999997), point_t(186.30000000000013, 132.29999999999998), point_t(185.0000000000001, 131.49999999999997), point_t(184.0000000000001, 131.49999999999997), point_t(183.5000000000001, 131.49999999999997), point_t(184.2000000000001, 132.19999999999996), point_t(184.5000000000001, 133.49999999999997), point_t(184.5000000000001, 134.49999999999997), point_t(184.5000000000001, 135.49999999999997), point_t(184.5000000000001, 136.49999999999997), point_t(185.2000000000001, 137.19999999999996), point_t(185.5000000000001, 138.49999999999997), point_t(186.2000000000001, 139.19999999999996), point_t(186.2000000000001, 139.19999999999996), point_t(186.5000000000001, 140.49999999999997), point_t(186.5000000000001, 141.99999999999997), point_t(186.5000000000001, 142.49999999999997), point_t(187.2000000000001, 143.19999999999996), point_t(187.5000000000001, 144.49999999999997), point_t(187.5000000000001, 145.49999999999997), point_t(188.1000000000001, 145.79999999999998), point_t(188.5000000000001, 146.49999999999997), point_t(189.2000000000001, 147.19999999999996), point_t(189.5000000000001, 148.99999999999997), point_t(189.5000000000001, 149.49999999999997), point_t(189.80000000000013, 150.59999999999997), point_t(189.0000000000001, 151.09999999999997), point_t(188.5000000000001, 151.49999999999997), point_t(187.1000000000001, 150.89999999999998), point_t(186.8000000000001, 149.99999999999997), point_t(186.50000000000009, 148.99999999999997), point_t(186.50000000000009, 148.49999999999997), point_t(185.8000000000001, 147.79999999999998), point_t(185.50000000000009, 146.49999999999997), point_t(184.8000000000001, 145.79999999999998), point_t(184.50000000000009, 144.99999999999997), point_t(184.50000000000009, 144.49999999999997), point_t(183.50000000000009, 142.99999999999997), point_t(183.50000000000009, 142.49999999999997), point_t(182.8000000000001, 141.79999999999998), point_t(182.50000000000009, 140.49999999999997), point_t(182.50000000000009, 139.49999999999997), point_t(182.50000000000009, 138.49999999999997), point_t(181.8000000000001, 137.79999999999998), point_t(181.50000000000009, 136.99999999999997), point_t(181.50000000000009, 136.49999999999997), point_t(180.8000000000001, 135.79999999999998), point_t(180.50000000000009, 133.99999999999997), point_t(180.50000000000009, 133.49999999999997), point_t(179.8000000000001, 132.79999999999998), point_t(179.50000000000009, 131.49999999999997), point_t(178.8000000000001, 130.79999999999998), point_t(178.50000000000009, 129.99999999999997), point_t(178.00000000000009, 129.49999999999997), point_t(177.3000000000001, 129.29999999999998), point_t(176.00000000000009, 128.49999999999997), point_t(175.3000000000001, 128.29999999999998), point_t(174.50000000000009, 127.49999999999999), point_t(173.50000000000009, 127.49999999999999), point_t(172.50000000000009, 127.49999999999999), point_t(171.60000000000008, 127.19999999999999), point_t(171.10000000000008, 126.89999999999999), point_t(170.50000000000009, 125.99999999999999), point_t(169.50000000000009, 125.49999999999999), point_t(168.3000000000001, 125.29999999999998), point_t(167.50000000000009, 124.49999999999999), point_t(166.50000000000009, 124.49999999999999), point_t(165.60000000000008, 124.39999999999999), point_t(165.30000000000007, 124.3), point_t(164.00000000000006, 123.5), point_t(163.30000000000007, 123.3), point_t(162.50000000000006, 122.5), point_t(162.00000000000006, 122.5), point_t(161.10000000000005, 121.9), point_t(160.00000000000006, 120.5), point_t(159.50000000000006, 120.5), point_t(158.30000000000007, 120.3), point_t(157.50000000000006, 119.5), point_t(157.00000000000006, 119.5), point_t(156.30000000000007, 119.3), point_t(154.90000000000006, 118.2), point_t(154.50000000000006, 117.5), point_t(153.90000000000006, 117.2), point_t(152.90000000000006, 116.2), point_t(152.50000000000006, 115.5), point_t(151.30000000000007, 115.3), point_t(150.60000000000008, 114.6), point_t(149.9000000000001, 114.19999999999999), point_t(149.50000000000009, 113.49999999999999), point_t(148.50000000000009, 112.49999999999999), point_t(147.9000000000001, 112.19999999999999), point_t(147.50000000000009, 111.49999999999999), point_t(146.50000000000009, 110.49999999999999), point_t(145.9000000000001, 110.19999999999999), point_t(145.50000000000009, 109.49999999999999), point_t(144.8000000000001, 108.79999999999998), point_t(144.50000000000009, 107.49999999999999), point_t(143.8000000000001, 106.79999999999998), point_t(143.50000000000009, 105.99999999999999), point_t(143.50000000000009, 105.49999999999999), point_t(142.8000000000001, 104.79999999999998), point_t(142.50000000000009, 103.49999999999999), point_t(141.8000000000001, 102.79999999999998), point_t(141.50000000000009, 101.49999999999999), point_t(141.50000000000009, 100.49999999999999), point_t(141.50000000000009, 99.49999999999999), point_t(140.8000000000001, 98.79999999999998), point_t(140.50000000000009, 97.99999999999999), point_t(140.50000000000009, 97.49999999999999), point_t(139.8000000000001, 96.79999999999998), point_t(139.50000000000009, 95.49999999999999), point_t(138.9000000000001, 95.19999999999999), point_t(138.00000000000009, 94.49999999999999), point_t(137.50000000000009, 94.49999999999999), point_t(136.50000000000009, 94.49999999999999), point_t(135.10000000000008, 93.89999999999999), point_t(134.50000000000009, 92.99999999999999), point_t(133.9000000000001, 92.19999999999999), point_t(133.50000000000009, 91.49999999999999), point_t(132.9000000000001, 91.19999999999999), point_t(131.9000000000001, 90.19999999999999), point_t(131.50000000000009, 89.49999999999999), point_t(130.8000000000001, 88.79999999999998), point_t(130.50000000000009, 87.49999999999999), point_t(129.50000000000009, 87.49999999999999), point_t(128.8000000000001, 88.19999999999999), point_t(128.8000000000001, 88.19999999999999), point_t(127.5000000000001, 88.49999999999999), point_t(126.0000000000001, 88.49999999999999), point_t(125.5000000000001, 88.49999999999999), point_t(124.8000000000001, 89.19999999999999), point_t(123.3000000000001, 89.19999999999999), point_t(122.5000000000001, 88.49999999999999), point_t(121.5000000000001, 88.49999999999999), point_t(120.5000000000001, 88.49999999999999), point_t(119.6000000000001, 88.39999999999999), point_t(119.3000000000001, 88.3), point_t(118.0000000000001, 87.5), point_t(117.5000000000001, 87.5), point_t(116.3000000000001, 87.3), point_t(115.5000000000001, 86.5), point_t(114.5000000000001, 86.5), point_t(114.5000000000001, 87.5), point_t(114.3000000000001, 88.7), point_t(113.5000000000001, 89.5), point_t(113.5000000000001, 90.5), point_t(114.2000000000001, 91.2), point_t(114.4000000000001, 92.2), point_t(114.5000000000001, 92.5), point_t(114.5000000000001, 93.5), point_t(114.5000000000001, 94.5), point_t(114.5000000000001, 95.5), point_t(114.5000000000001, 96.5), point_t(114.5000000000001, 97.5), point_t(114.5000000000001, 98.5), point_t(114.5000000000001, 100.0), point_t(114.5000000000001, 100.5), point_t(115.2000000000001, 101.2), point_t(115.5000000000001, 102.5), point_t(115.5000000000001, 103.5), point_t(115.5000000000001, 104.5), point_t(115.5000000000001, 105.5), point_t(115.5000000000001, 106.5), point_t(115.5000000000001, 107.5), point_t(115.2000000000001, 108.4), point_t(114.9000000000001, 108.9), point_t(113.5000000000001, 109.5), point_t(112.5000000000001, 109.5), point_t(111.8000000000001, 110.2), point_t(110.5000000000001, 110.5), point_t(109.5000000000001, 110.5), point_t(108.5000000000001, 110.5), point_t(107.5000000000001, 110.5), point_t(106.7000000000001, 111.0), point_t(106.5000000000001, 111.1), point_t(105.0000000000001, 110.5), point_t(104.3000000000001, 110.3), point_t(103.5000000000001, 109.5), point_t(102.5000000000001, 109.5), point_t(101.5000000000001, 109.5), point_t(100.5000000000001, 109.5), point_t(99.3000000000001, 109.3), point_t(98.6000000000001, 108.6), point_t(98.0000000000001, 108.5), point_t(97.5000000000001, 108.5), point_t(96.3000000000001, 108.3), point_t(95.0000000000001, 107.5), point_t(94.5000000000001, 107.5), point_t(93.5000000000001, 107.5), point_t(92.5000000000001, 107.5), point_t(91.5000000000001, 107.5), point_t(90.5000000000001, 107.5), point_t(89.5000000000001, 107.5), point_t(88.3000000000001, 107.3), point_t(87.5000000000001, 106.5), point_t(86.8000000000001, 107.2), point_t(85.8000000000001, 107.2), point_t(85.3000000000001, 107.2), point_t(84.0000000000001, 106.5), point_t(83.5000000000001, 106.5), point_t(82.8000000000001, 107.2), point_t(81.5000000000001, 107.5), point_t(80.5000000000001, 107.5), point_t(79.5000000000001, 107.5), point_t(78.8000000000001, 108.2), point_t(77.5000000000001, 108.5), point_t(76.7000000000001, 109.0), point_t(76.5000000000001, 109.1), point_t(75.0000000000001, 108.5), point_t(74.5000000000001, 108.5), point_t(73.8000000000001, 109.2), point_t(72.5000000000001, 109.5), point_t(71.5000000000001, 109.5), point_t(70.5000000000001, 109.5), point_t(69.5000000000001, 109.5), point_t(68.5000000000001, 109.5), point_t(67.5000000000001, 109.5), point_t(67.5000000000001, 110.0), point_t(66.7000000000001, 110.8), point_t(65.5000000000001, 110.5), point_t(64.9000000000001, 110.2), point_t(64.0000000000001, 109.5), point_t(63.3000000000001, 109.3), point_t(62.0000000000001, 108.5), point_t(61.5000000000001, 108.5), point_t(60.5000000000001, 108.5), point_t(59.5000000000001, 108.5), point_t(58.3000000000001, 108.3), point_t(57.5000000000001, 107.5), point_t(56.8000000000001, 108.2), point_t(55.8000000000001, 108.4), point_t(55.5000000000001, 108.5), point_t(54.3000000000001, 108.3), point_t(53.0000000000001, 107.5), point_t(52.5000000000001, 107.5), point_t(51.8000000000001, 108.2), point_t(50.3000000000001, 108.2), point_t(49.600000000000094, 107.60000000000001), point_t(49.00000000000009, 107.50000000000001), point_t(48.50000000000009, 107.50000000000001), point_t(47.80000000000009, 108.20000000000002), point_t(46.50000000000009, 108.50000000000001), point_t(45.50000000000009, 108.50000000000001), point_t(44.30000000000009, 108.30000000000001), point_t(43.50000000000009, 107.50000000000001), point_t(42.50000000000009, 107.50000000000001), point_t(41.50000000000009, 107.50000000000001), point_t(40.50000000000009, 107.50000000000001), point_t(39.50000000000009, 107.50000000000001), point_t(38.50000000000009, 107.50000000000001), point_t(37.50000000000009, 107.50000000000001), point_t(36.50000000000009, 107.50000000000001), point_t(35.50000000000009, 107.50000000000001), point_t(34.50000000000009, 107.50000000000001), point_t(33.50000000000009, 107.50000000000001), point_t(32.50000000000009, 107.50000000000001), point_t(31.500000000000092, 107.50000000000001), point_t(30.500000000000092, 107.50000000000001), point_t(29.500000000000092, 107.50000000000001), point_t(28.500000000000092, 107.50000000000001), point_t(27.500000000000092, 107.50000000000001), point_t(26.500000000000092, 107.50000000000001), point_t(25.500000000000092, 107.50000000000001), point_t(24.500000000000092, 107.50000000000001), point_t(23.300000000000093, 107.30000000000001), point_t(22.600000000000094, 106.60000000000001), point_t(22.000000000000092, 106.50000000000001), point_t(21.500000000000092, 106.50000000000001), point_t(20.500000000000092, 106.50000000000001), point_t(19.500000000000092, 106.50000000000001), point_t(18.300000000000093, 106.30000000000001), point_t(17.000000000000092, 105.50000000000001), point_t(16.500000000000092, 105.50000000000001), point_t(15.500000000000092, 106.50000000000001), point_t(14.800000000000093, 107.20000000000002), point_t(13.800000000000093, 107.40000000000002), point_t(13.500000000000092, 107.50000000000001), point_t(12.500000000000092, 107.50000000000001), point_t(11.500000000000092, 107.50000000000001), point_t(10.500000000000092, 107.50000000000001), point_t(9.100000000000092, 106.90000000000002), point_t(9.100000000000092, 105.90000000000002), point_t(9.100000000000092, 105.10000000000002), point_t(10.000000000000092, 104.80000000000003), point_t(10.500000000000092, 104.50000000000003), point_t(12.000000000000092, 104.50000000000003), point_t(12.500000000000092, 104.50000000000003), point_t(13.200000000000092, 103.80000000000003), point_t(14.500000000000092, 103.50000000000003), point_t(15.500000000000092, 103.50000000000003), point_t(16.500000000000092, 103.50000000000003), point_t(18.000000000000092, 103.50000000000003), point_t(18.500000000000092, 103.50000000000003), point_t(19.20000000000009, 102.80000000000003), point_t(20.500000000000092, 102.50000000000003), point_t(21.500000000000092, 102.50000000000003), point_t(22.500000000000092, 102.50000000000003), point_t(23.70000000000009, 102.70000000000003), point_t(24.40000000000009, 103.40000000000003), point_t(25.000000000000092, 103.50000000000003), point_t(25.500000000000092, 103.50000000000003), point_t(26.70000000000009, 103.70000000000003), point_t(28.000000000000092, 104.50000000000003), point_t(28.500000000000092, 104.50000000000003), point_t(29.500000000000092, 104.50000000000003), point_t(30.500000000000092, 104.50000000000003), point_t(31.500000000000092, 104.50000000000003), point_t(32.50000000000009, 104.50000000000003), point_t(34.00000000000009, 104.50000000000003), point_t(34.50000000000009, 104.50000000000003), point_t(35.200000000000095, 103.80000000000003), point_t(36.50000000000009, 103.50000000000003), point_t(37.50000000000009, 103.50000000000003), point_t(38.700000000000095, 103.70000000000003), point_t(39.4000000000001, 104.40000000000003), point_t(40.0000000000001, 104.50000000000003), point_t(40.5000000000001, 104.50000000000003), point_t(41.5000000000001, 104.50000000000003), point_t(42.5000000000001, 104.50000000000003), point_t(43.5000000000001, 104.50000000000003), point_t(44.5000000000001, 104.50000000000003), point_t(45.5000000000001, 104.50000000000003), point_t(47.0000000000001, 104.50000000000003), point_t(47.5000000000001, 104.50000000000003), point_t(48.2000000000001, 103.80000000000003), point_t(49.7000000000001, 103.80000000000003), point_t(50.5000000000001, 104.50000000000003), point_t(51.5000000000001, 104.50000000000003), point_t(52.5000000000001, 104.50000000000003), point_t(53.5000000000001, 104.50000000000003), point_t(54.5000000000001, 104.50000000000003), point_t(55.5000000000001, 104.50000000000003), point_t(56.5000000000001, 104.50000000000003), point_t(57.5000000000001, 104.50000000000003), point_t(58.2000000000001, 103.80000000000003), point_t(59.5000000000001, 103.50000000000003), point_t(60.4000000000001, 103.60000000000002), point_t(60.700000000000095, 103.70000000000002), point_t(62.00000000000009, 104.50000000000001), point_t(62.50000000000009, 104.50000000000001), point_t(63.200000000000095, 103.80000000000001), point_t(64.5000000000001, 103.50000000000001), point_t(65.5000000000001, 103.50000000000001), point_t(66.5000000000001, 103.50000000000001), point_t(66.5000000000001, 102.50000000000001), point_t(66.8000000000001, 101.60000000000001), point_t(67.1000000000001, 101.10000000000001), point_t(68.5000000000001, 100.50000000000001), point_t(69.5000000000001, 100.50000000000001), point_t(70.2000000000001, 99.80000000000001), point_t(71.2000000000001, 99.60000000000001), point_t(72.0000000000001, 99.50000000000001), point_t(72.5000000000001, 99.50000000000001), point_t(73.2000000000001, 98.80000000000001), point_t(74.5000000000001, 98.50000000000001), point_t(75.5000000000001, 98.50000000000001), point_t(76.5000000000001, 98.50000000000001), point_t(77.5000000000001, 98.50000000000001), point_t(78.5000000000001, 98.50000000000001), point_t(79.5000000000001, 98.50000000000001), point_t(80.5000000000001, 98.50000000000001), point_t(82.0000000000001, 98.50000000000001), point_t(82.5000000000001, 98.50000000000001), point_t(83.5000000000001, 97.50000000000001), point_t(83.8000000000001, 96.90000000000002), point_t(84.5000000000001, 96.50000000000001), point_t(85.5000000000001, 95.50000000000001), point_t(85.8000000000001, 94.90000000000002), point_t(86.5000000000001, 94.50000000000001), point_t(87.5000000000001, 93.50000000000001), point_t(87.8000000000001, 92.90000000000002), point_t(88.5000000000001, 92.50000000000001), point_t(89.5000000000001, 91.50000000000001), point_t(89.8000000000001, 90.90000000000002), point_t(90.5000000000001, 90.00000000000001), point_t(90.5000000000001, 89.50000000000001), point_t(90.7000000000001, 88.30000000000001), point_t(91.5000000000001, 87.50000000000001), point_t(91.5000000000001, 86.50000000000001), point_t(91.5000000000001, 85.50000000000001), point_t(90.9000000000001, 85.20000000000002), point_t(90.0000000000001, 84.50000000000001), point_t(89.3000000000001, 84.30000000000001), point_t(88.0000000000001, 83.50000000000001), point_t(87.5000000000001, 83.50000000000001), point_t(86.5000000000001, 83.50000000000001), point_t(85.5000000000001, 83.50000000000001), point_t(84.5000000000001, 83.50000000000001), point_t(83.5000000000001, 83.50000000000001), point_t(82.3000000000001, 83.30000000000001), point_t(81.5000000000001, 82.50000000000001), point_t(80.9000000000001, 82.20000000000002), point_t(80.0000000000001, 81.50000000000001), point_t(79.5000000000001, 81.50000000000001), point_t(78.5000000000001, 81.50000000000001), point_t(77.5000000000001, 81.50000000000001), point_t(76.5000000000001, 81.50000000000001), point_t(75.1000000000001, 80.90000000000002), point_t(74.5000000000001, 80.00000000000001), point_t(74.5000000000001, 79.50000000000001), point_t(73.5000000000001, 78.50000000000001), point_t(72.5000000000001, 78.50000000000001), point_t(71.5000000000001, 78.50000000000001), point_t(70.6000000000001, 78.40000000000002), point_t(70.3000000000001, 78.30000000000003), point_t(69.0000000000001, 77.50000000000003), point_t(68.1000000000001, 76.90000000000003), point_t(67.5000000000001, 76.00000000000003), point_t(67.0000000000001, 75.50000000000003), point_t(66.5000000000001, 75.50000000000003), point_t(65.5000000000001, 75.50000000000003), point_t(64.5000000000001, 75.50000000000003), point_t(63.3000000000001, 75.30000000000003), point_t(62.5000000000001, 74.50000000000003), point_t(61.5000000000001, 73.50000000000003), point_t(61.0000000000001, 73.50000000000003), point_t(60.3000000000001, 73.30000000000003), point_t(59.0000000000001, 72.50000000000003), point_t(58.3000000000001, 72.30000000000003), point_t(57.5000000000001, 71.50000000000003), point_t(56.3000000000001, 71.30000000000003), point_t(55.5000000000001, 70.50000000000003), point_t(54.8000000000001, 69.80000000000003), point_t(54.5000000000001, 69.00000000000003), point_t(54.5000000000001, 68.50000000000003), point_t(53.5000000000001, 67.50000000000003), point_t(53.0000000000001, 67.50000000000003), point_t(52.3000000000001, 67.30000000000003), point_t(51.5000000000001, 66.50000000000003), point_t(50.8000000000001, 65.80000000000003), point_t(50.5000000000001, 64.50000000000003), point_t(49.9000000000001, 64.20000000000003), point_t(48.9000000000001, 63.20000000000003), point_t(48.5000000000001, 62.50000000000003), point_t(47.9000000000001, 62.20000000000003), point_t(47.5000000000001, 61.50000000000003), point_t(46.5000000000001, 60.00000000000003), point_t(46.5000000000001, 59.50000000000003), point_t(45.5000000000001, 58.50000000000003), point_t(44.9000000000001, 58.20000000000003), point_t(44.0000000000001, 57.50000000000003), point_t(43.3000000000001, 57.300000000000026), point_t(42.5000000000001, 56.50000000000003), point_t(41.9000000000001, 55.50000000000003), point_t(42.4000000000001, 54.70000000000003), point_t(42.5000000000001, 54.00000000000003), point_t(42.5000000000001, 53.50000000000003), point_t(42.5000000000001, 52.50000000000003), point_t(42.5000000000001, 51.50000000000003), point_t(42.5000000000001, 50.50000000000003), point_t(42.5000000000001, 49.50000000000003), point_t(42.5000000000001, 48.50000000000003), point_t(42.5000000000001, 47.50000000000003), point_t(42.5000000000001, 46.50000000000003), point_t(42.5000000000001, 45.50000000000003), point_t(42.5000000000001, 44.50000000000003), point_t(42.5000000000001, 43.50000000000003), point_t(42.5000000000001, 42.50000000000003), point_t(42.7000000000001, 41.300000000000026), point_t(43.5000000000001, 40.50000000000003), point_t(44.5000000000001, 39.50000000000003), point_t(44.8000000000001, 38.90000000000003), point_t(45.5000000000001, 38.50000000000003), point_t(46.5000000000001, 37.50000000000003), point_t(46.8000000000001, 36.90000000000003), point_t(47.5000000000001, 36.50000000000003), point_t(48.2000000000001, 35.800000000000026), point_t(49.5000000000001, 35.50000000000003), point_t(49.8000000000001, 34.90000000000003), point_t(50.5000000000001, 34.00000000000003), point_t(50.5000000000001, 33.50000000000003), point_t(50.7000000000001, 32.300000000000026), point_t(51.5000000000001, 31.500000000000025), point_t(52.5000000000001, 30.900000000000023), point_t(53.5000000000001, 31.500000000000025), point_t(54.1000000000001, 31.800000000000026), point_t(55.0000000000001, 32.50000000000003), point_t(56.0000000000001, 32.50000000000003), point_t(56.5000000000001, 32.50000000000003), point_t(57.2000000000001, 31.80000000000003), point_t(58.5000000000001, 31.50000000000003), point_t(59.5000000000001, 31.50000000000003), point_t(60.5000000000001, 30.900000000000027), point_t(61.5000000000001, 31.50000000000003), point_t(62.5000000000001, 31.50000000000003), point_t(63.5000000000001, 31.50000000000003), point_t(64.2000000000001, 30.80000000000003), point_t(64.2000000000001, 30.80000000000003), point_t(65.7000000000001, 30.80000000000003), point_t(66.5000000000001, 31.50000000000003), point_t(67.5000000000001, 31.50000000000003), point_t(68.5000000000001, 31.50000000000003), point_t(69.5000000000001, 31.50000000000003), point_t(70.5000000000001, 30.900000000000027), point_t(71.5000000000001, 31.50000000000003), point_t(72.3000000000001, 31.00000000000003), point_t(72.5000000000001, 30.900000000000027), point_t(74.0000000000001, 31.50000000000003), point_t(74.7000000000001, 31.700000000000028), point_t(75.5000000000001, 32.50000000000003), point_t(76.5000000000001, 32.50000000000003), point_t(77.5000000000001, 32.50000000000003), point_t(78.5000000000001, 32.50000000000003), point_t(79.5000000000001, 32.50000000000003), point_t(80.5000000000001, 32.50000000000003), point_t(81.5000000000001, 32.50000000000003), point_t(82.4000000000001, 32.800000000000026), point_t(82.9000000000001, 33.10000000000002), point_t(83.5000000000001, 34.50000000000002), point_t(83.5000000000001, 35.50000000000002), point_t(84.1000000000001, 35.80000000000002), point_t(85.0000000000001, 36.50000000000002), point_t(85.5000000000001, 36.50000000000002), point_t(86.5000000000001, 36.50000000000002), point_t(87.5000000000001, 36.50000000000002), point_t(89.0000000000001, 36.50000000000002), point_t(89.5000000000001, 36.50000000000002), point_t(88.8000000000001, 35.80000000000002), point_t(88.5000000000001, 34.50000000000002), point_t(87.8000000000001, 33.80000000000002), point_t(87.5000000000001, 32.50000000000002), point_t(87.0000000000001, 32.50000000000002), point_t(86.3000000000001, 32.30000000000002), point_t(85.5000000000001, 31.500000000000018), point_t(84.8000000000001, 30.80000000000002), point_t(84.5000000000001, 29.500000000000018), point_t(84.0000000000001, 29.500000000000018), point_t(83.1000000000001, 28.900000000000016), point_t(82.5000000000001, 27.500000000000018), point_t(82.0000000000001, 27.500000000000018), point_t(81.3000000000001, 27.30000000000002), point_t(79.90000000000009, 26.200000000000017), point_t(79.50000000000009, 25.500000000000018), point_t(78.30000000000008, 25.30000000000002), point_t(77.50000000000009, 24.500000000000018), point_t(76.90000000000009, 24.200000000000017), point_t(76.50000000000009, 23.500000000000018), point_t(75.50000000000009, 22.500000000000018), point_t(74.80000000000008, 21.80000000000002), point_t(74.50000000000009, 21.000000000000018), point_t(74.50000000000009, 20.500000000000018), point_t(73.50000000000009, 19.500000000000018), point_t(73.00000000000009, 19.500000000000018), point_t(72.30000000000008, 19.30000000000002), point_t(71.50000000000009, 18.500000000000018), point_t(70.50000000000009, 17.000000000000018), point_t(70.50000000000009, 16.500000000000018), point_t(69.80000000000008, 15.800000000000018), point_t(69.50000000000009, 14.500000000000018), point_t(69.00000000000009, 14.500000000000018), point_t(68.30000000000008, 14.300000000000018), point_t(67.00000000000009, 13.500000000000018), point_t(66.30000000000008, 13.300000000000018), point_t(65.50000000000009, 12.500000000000018), point_t(64.50000000000009, 11.500000000000018), point_t(63.500000000000085, 11.500000000000018), point_t(62.60000000000009, 11.400000000000018), point_t(62.30000000000009, 11.300000000000018), point_t(61.00000000000009, 10.500000000000018), point_t(60.50000000000009, 10.500000000000018), point_t(59.30000000000009, 10.300000000000018), point_t(58.50000000000009, 9.500000000000018), point_t(57.90000000000009, 9.200000000000017), point_t(57.00000000000009, 8.500000000000018), point_t(56.30000000000009, 8.300000000000018), point_t(55.50000000000009, 7.500000000000019), point_t(54.50000000000009, 7.500000000000019), point_t(53.30000000000009, 7.3000000000000185), point_t(52.50000000000009, 6.500000000000019), point_t(51.90000000000009, 6.200000000000019), point_t(51.50000000000009, 5.500000000000019), point_t(50.80000000000009, 4.8000000000000185), point_t(50.90000000000009, 3.8000000000000185), point_t(51.100000000000094, 3.1000000000000183), point_t(52.700000000000095, 2.8000000000000185), point_t(53.50000000000009, 3.5000000000000187), point_t(54.50000000000009, 3.5000000000000187), point_t(55.50000000000009, 3.5000000000000187), point_t(56.40000000000009, 3.6000000000000187), point_t(56.70000000000009, 3.700000000000019), point_t(58.10000000000009, 4.8000000000000185), point_t(58.500000000000085, 5.500000000000019), point_t(59.500000000000085, 5.500000000000019), point_t(60.70000000000009, 5.700000000000019), point_t(61.500000000000085, 6.500000000000019), point_t(62.000000000000085, 6.500000000000019), point_t(62.70000000000009, 6.700000000000019), point_t(63.500000000000085, 7.500000000000019), point_t(64.50000000000009, 8.500000000000018), point_t(65.50000000000009, 8.500000000000018), point_t(66.40000000000009, 8.600000000000017), point_t(66.70000000000009, 8.700000000000017), point_t(68.00000000000009, 9.500000000000018), point_t(68.50000000000009, 9.500000000000018), point_t(69.70000000000009, 9.700000000000017), point_t(71.00000000000009, 10.500000000000018), point_t(71.70000000000009, 10.700000000000017), point_t(72.50000000000009, 11.500000000000018), point_t(73.10000000000008, 11.800000000000018), point_t(74.00000000000009, 12.500000000000018), point_t(74.70000000000009, 12.700000000000017), point_t(75.50000000000009, 13.500000000000018), point_t(76.70000000000009, 13.700000000000017), point_t(77.50000000000009, 14.500000000000018), point_t(78.10000000000008, 14.800000000000018), point_t(78.50000000000009, 15.500000000000018), point_t(79.50000000000009, 16.500000000000018), point_t(80.10000000000008, 16.80000000000002), point_t(80.50000000000009, 17.500000000000018), point_t(81.50000000000009, 18.500000000000018), point_t(82.00000000000009, 18.500000000000018), point_t(82.70000000000009, 18.700000000000017), point_t(83.50000000000009, 19.500000000000018), point_t(84.20000000000009, 20.200000000000017), point_t(84.50000000000009, 21.500000000000018), point_t(85.10000000000008, 21.80000000000002), point_t(86.00000000000009, 22.500000000000018), point_t(86.70000000000009, 22.700000000000017), point_t(87.50000000000009, 23.500000000000018), point_t(88.50000000000009, 24.500000000000018), point_t(89.10000000000008, 24.80000000000002), point_t(89.50000000000009, 25.500000000000018), point_t(90.50000000000009, 26.500000000000018), point_t(91.10000000000008, 26.80000000000002), point_t(91.50000000000009, 27.500000000000018), point_t(92.20000000000009, 28.200000000000017), point_t(92.50000000000009, 29.500000000000018), point_t(93.10000000000008, 29.80000000000002), point_t(93.50000000000009, 30.500000000000018), point_t(94.50000000000009, 31.500000000000018), point_t(95.10000000000008, 31.80000000000002), point_t(96.00000000000009, 32.50000000000002), point_t(96.90000000000009, 33.10000000000002), point_t(97.50000000000009, 34.00000000000002), point_t(97.50000000000009, 34.50000000000002), point_t(98.50000000000009, 35.50000000000002), point_t(99.10000000000008, 35.80000000000002), point_t(99.50000000000009, 36.50000000000002), point_t(100.50000000000009, 37.50000000000002), point_t(101.10000000000008, 37.80000000000002), point_t(101.50000000000009, 38.50000000000002), point_t(102.50000000000009, 39.50000000000002), point_t(103.20000000000009, 40.200000000000024), point_t(103.40000000000009, 41.200000000000024), point_t(103.50000000000009, 42.00000000000002), point_t(104.00000000000009, 42.50000000000002), point_t(104.90000000000009, 43.10000000000002), point_t(105.20000000000009, 44.00000000000002), point_t(105.50000000000009, 44.50000000000002), point_t(105.50000000000009, 46.00000000000002), point_t(106.00000000000009, 46.50000000000002), point_t(106.90000000000009, 47.10000000000002), point_t(107.20000000000009, 48.00000000000002), point_t(107.50000000000009, 49.00000000000002), point_t(107.50000000000009, 49.50000000000002), point_t(108.50000000000009, 50.50000000000002), point_t(109.10000000000008, 50.80000000000002), point_t(110.00000000000009, 51.50000000000002), point_t(110.70000000000009, 51.700000000000024), point_t(111.50000000000009, 52.50000000000002), point_t(112.00000000000009, 52.50000000000002), point_t(112.70000000000009, 52.700000000000024), point_t(113.50000000000009, 53.50000000000002), point_t(114.50000000000009, 55.00000000000002), point_t(114.50000000000009, 55.50000000000002), point_t(115.50000000000009, 56.50000000000002), point_t(116.10000000000008, 56.80000000000002), point_t(116.50000000000009, 57.50000000000002), point_t(117.50000000000009, 58.50000000000002), point_t(118.50000000000009, 58.50000000000002), point_t(119.50000000000009, 58.50000000000002), point_t(120.50000000000009, 58.50000000000002), point_t(121.50000000000009, 58.50000000000002), point_t(122.50000000000009, 58.50000000000002), point_t(123.50000000000009, 58.50000000000002), point_t(124.50000000000009, 58.50000000000002), point_t(124.50000000000009, 58.00000000000002), point_t(125.10000000000008, 57.10000000000002), point_t(126.00000000000009, 56.50000000000002), point_t(126.80000000000008, 55.90000000000002), point_t(127.50000000000009, 55.50000000000002), point_t(127.80000000000008, 54.90000000000002), point_t(128.80000000000007, 53.90000000000002), point_t(129.50000000000006, 53.50000000000002), point_t(129.80000000000007, 52.90000000000002), point_t(130.80000000000007, 51.90000000000002), point_t(131.50000000000006, 51.50000000000002), point_t(131.80000000000007, 50.90000000000002), point_t(132.50000000000006, 50.00000000000002), point_t(133.10000000000005, 49.10000000000002), point_t(134.00000000000006, 48.50000000000002), point_t(134.80000000000007, 47.90000000000002), point_t(135.50000000000006, 47.50000000000002), point_t(135.80000000000007, 46.90000000000002), point_t(136.80000000000007, 45.90000000000002), point_t(137.50000000000006, 45.50000000000002), point_t(137.80000000000007, 44.90000000000002), point_t(138.50000000000006, 44.00000000000002), point_t(138.70000000000005, 43.30000000000002), point_t(139.50000000000006, 42.50000000000002), point_t(139.80000000000007, 41.90000000000002), point_t(140.50000000000006, 41.50000000000002), point_t(142.00000000000006, 40.50000000000002), point_t(142.50000000000006, 40.00000000000002), point_t(142.70000000000005, 39.30000000000002), point_t(143.50000000000006, 38.50000000000002), point_t(143.50000000000006, 37.50000000000002), point_t(143.50000000000006, 36.50000000000002), point_t(142.90000000000006, 35.50000000000002), point_t(143.40000000000006, 34.700000000000024), point_t(143.50000000000006, 34.50000000000002), point_t(142.80000000000007, 33.80000000000002), point_t(142.50000000000006, 32.50000000000002), point_t(142.50000000000006, 31.50000000000002), point_t(142.50000000000006, 30.50000000000002), point_t(142.50000000000006, 29.50000000000002), point_t(142.50000000000006, 28.00000000000002), point_t(142.50000000000006, 27.50000000000002), point_t(141.80000000000007, 26.800000000000022), point_t(141.50000000000006, 25.50000000000002), point_t(141.50000000000006, 24.50000000000002), point_t(141.70000000000005, 23.300000000000022), point_t(142.40000000000003, 22.600000000000023), point_t(142.50000000000003, 22.00000000000002), point_t(142.50000000000003, 21.50000000000002), point_t(142.50000000000003, 20.00000000000002), point_t(142.50000000000003, 19.50000000000002), point_t(141.80000000000004, 18.800000000000022), point_t(141.50000000000003, 17.50000000000002), point_t(141.50000000000003, 16.50000000000002), point_t(141.50000000000003, 15.500000000000021), point_t(141.50000000000003, 14.500000000000021), point_t(141.50000000000003, 13.500000000000021), point_t(141.50000000000003, 12.500000000000021), point_t(141.50000000000003, 11.500000000000021), point_t(141.50000000000003, 10.500000000000021), point_t(141.70000000000002, 9.300000000000022), point_t(142.50000000000003, 8.500000000000021), point_t(142.50000000000003, 7.500000000000021), point_t(143.30000000000004, 6.500000000000021), point_t(143.30000000000004, 5.600000000000021), point_t(143.30000000000004, 5.200000000000021), point_t(144.10000000000005, 5.700000000000021), point_t(145.10000000000005, 6.50000000000002), point_t(144.80000000000004, 7.400000000000021), point_t(144.50000000000003, 8.500000000000021), point_t(144.30000000000004, 9.70000000000002), point_t(143.50000000000003, 10.500000000000021), point_t(144.20000000000002, 11.20000000000002), point_t(144.20000000000002, 11.20000000000002), point_t(144.50000000000003, 12.500000000000021), point_t(144.50000000000003, 13.500000000000021), point_t(144.50000000000003, 14.500000000000021), point_t(144.50000000000003, 15.500000000000021), point_t(144.50000000000003, 16.50000000000002), point_t(144.50000000000003, 18.00000000000002), point_t(144.50000000000003, 18.50000000000002), point_t(145.20000000000002, 19.20000000000002), point_t(145.50000000000003, 20.50000000000002), point_t(145.50000000000003, 21.50000000000002), point_t(145.50000000000003, 22.50000000000002), point_t(145.50000000000003, 23.50000000000002), point_t(146.20000000000002, 24.20000000000002), point_t(146.4, 25.20000000000002), point_t(146.5, 25.50000000000002), point_t(146.5, 26.50000000000002), point_t(146.5, 27.50000000000002), point_t(146.5, 28.50000000000002), point_t(146.5, 29.50000000000002), point_t(146.5, 30.50000000000002), point_t(146.5, 32.00000000000002), point_t(146.5, 32.50000000000002), point_t(147.2, 33.200000000000024), point_t(147.5, 34.50000000000002), point_t(147.5, 35.50000000000002), point_t(147.5, 36.50000000000002), point_t(147.5, 37.50000000000002), point_t(147.5, 38.50000000000002), point_t(147.5, 39.50000000000002), point_t(147.5, 40.50000000000002), point_t(147.5, 41.50000000000002), point_t(147.3, 42.700000000000024), point_t(146.5, 43.50000000000002), point_t(146.5, 44.50000000000002), point_t(146.3, 45.700000000000024), point_t(145.60000000000002, 46.40000000000003), point_t(145.20000000000002, 47.10000000000003), point_t(144.50000000000003, 47.50000000000003), point_t(143.50000000000003, 48.50000000000003), point_t(143.20000000000002, 49.10000000000003), point_t(142.50000000000003, 49.50000000000003), point_t(141.50000000000003, 50.50000000000003), point_t(140.80000000000004, 51.20000000000003), point_t(140.00000000000003, 51.50000000000003), point_t(139.50000000000003, 52.00000000000003), point_t(139.30000000000004, 52.70000000000003), point_t(138.50000000000003, 53.50000000000003), point_t(137.80000000000004, 54.20000000000003), point_t(137.00000000000003, 54.50000000000003), point_t(136.20000000000002, 55.10000000000003), point_t(135.50000000000003, 55.50000000000003), point_t(135.20000000000002, 56.10000000000003), point_t(134.20000000000002, 57.10000000000003), point_t(133.50000000000003, 58.00000000000003), point_t(133.00000000000003, 58.800000000000026), point_t(133.20000000000002, 59.700000000000024), point_t(134.00000000000003, 59.50000000000002), point_t(134.50000000000003, 59.00000000000002)};
    }

    std::vector<std::vector<point_t>> reconstructed_fish() {
        return {
            {point_t(-22.88290454273092, 1.339908989669631),
                point_t(-22.80724393876914, 1.768765208339534),
                point_t(-22.63974661314057, 2.1904992337901126),
                point_t(-22.381087018458373, 2.603412891949273),
                point_t(-22.032306685618043, 3.0058435252509126),
                point_t(-21.594810029920094, 3.3961706875701076),
                point_t(-21.070358695985288, 3.7728226691877405),
                point_t(-20.461064464233562, 4.134282825510806),
                point_t(-19.76938074748957, 4.479095684064902),
                point_t(-18.99809271195519, 4.805872805168178),
                point_t(-18.1503060623282, 5.113298372687904),
                point_t(-17.229434536225646, 5.400134492367613),
                point_t(-16.239186158267273, 5.6652261763903775),
                point_t(-15.183548309169119, 5.907505994107097),
                point_t(-14.066771669968702, 6.125998370202976),
                point_t(-12.893353106032732, 6.319823512995),
                point_t(-11.668017559767405, 6.488200957042521),
                point_t(-10.395699024942985, 6.630452705806132),
                point_t(-9.081520679242319, 6.746005961700427),
                point_t(-7.730774255032474, 6.834395432547723),
                point_t(-6.34889873142621, 6.8952652051454315),
                point_t(-4.941458433432749, 6.928370178402907),
                point_t(-3.514120626384807, 6.933577050277036),
                point_t(-2.072632695861395, 6.910864854532514),
                point_t(-0.6227990049945509, 6.860325045165479),
                point_t(0.829542477652389, 6.782161128150538),
                point_t(2.2785436855236663, 6.676687841994044),
                point_t(3.7183700021711354, 6.544329890393208),
                point_t(5.143223755195389, 6.385620232104217),
                point_t(6.54736756142623, 6.201197934905414),
                point_t(7.925147429339807, 5.991805602296914),
                point_t(9.27101552568683, 5.758286383298416),
                point_t(10.57955251465907, 5.501580577385674),
                point_t(11.845489379641922, 5.222721848236382),
                point_t(13.063728639684618, 4.922833061531375),
                point_t(14.229364875256916, 4.603121763570837),
                point_t(15.337704480642222, 4.264875318911541),
                point_t(16.384284563431216, 3.909455726604156),
                point_t(17.364890915014403, 3.5382941359037767),
                point_t(18.27557497971288, 3.152885083536977),
                point_t(19.112669754218363, 2.7547804757299175),
                point_t(19.8728045533213, 2.3455833392297007),
                point_t(20.552918582470546, 1.9269413664814294),
                point_t(21.15027326251273, 1.500540280952282),
                point_t(21.662463256983933, 1.0680970493180826),
                point_t(22.08742615755068, 0.6313529678444796),
                point_t(22.42345078860041, 0.19206665080145946),
                point_t(22.66918409754167, -0.24799305085560985),
                point_t(22.823636603069396, -0.6870541720244798),
                point_t(22.886186379456888, -1.123348768534102),
                point_t(22.85658156083118, -1.5551200360317854),
                point_t(22.73494135534804, -1.9806293840142954),
                point_t(22.5217555651828, -2.398163436518176),
                point_t(22.21788261426986, -2.806040931280039),
                point_t(21.82454609173248, -3.202619489586303),
                point_t(21.34332982492124, -3.586302229552592),
                point_t(20.776171501900308, -3.9555441962035034),
                point_t(20.125354869061496, -4.3088585824609735),
                point_t(19.39350053528358, -4.644822715991658),
                point_t(18.583555419665224, -4.962083787806288),
                point_t(17.698780885321977, -5.259364299543969),
                point_t(16.742739607027964, -5.535467207507246),
                point_t(15.719281225582069, -5.789280742734547),
                point_t(14.632526846663099, -6.019782887701424),
                point_t(13.486852446591811, -6.226045491624382),
                point_t(12.286871251818626, -6.407238007796531),
                point_t(11.037415163088994, -6.562630837906014),
                point_t(9.743515299084525, -6.691598269870911),
                point_t(8.410381737883636, -6.793620997360951),
                point_t(7.043382537816154, -6.868288210860814),
                point_t(5.648022122186826, -6.915299251855069),
                point_t(4.229919114905686, -6.934464823473883),
                point_t(2.7947837162724136, -6.925707752724718),
                point_t(1.348394710015191, -6.889063301240726),
                point_t(-0.10342380583173238, -6.824679023294602),
                point_t(-1.5548258705153593, -6.732814171649607),
                point_t(-2.9999672001845235, -6.613838653640187),
                point_t(-4.433028720758275, -6.468231541685698),
                point_t(-5.848239999283415, -6.296579144234857),
                point_t(-7.239902479431288, -6.099572644908558),
                point_t(-8.602412427572977, -5.878005319347366),
                point_t(-9.930283497036672, -5.632769340970517),
                point_t(-11.218168819689485, -5.364852188508419),
                point_t(-12.46088253588819, -5.0753326697743475),
                point_t(-13.65342067610572, -4.765376577686135),
                point_t(-14.790981310150338, -4.436231996029546),
                point_t(-15.86898388284365, -4.089224273865391),
                point_t(-16.883087658299747, -3.7257506888166),
                point_t(-17.82920919853644, -3.34727482072444),
                point_t(-18.703538806038917, -2.955320658329013),
                point_t(-19.502555864066952, -2.5514664627045063),
                point_t(-20.22304301293596, -2.1373384121588783),
                point_t(-20.862099105188896, -1.7146040541877545),
                point_t(-21.417150887493115, -1.2849655908492394),
                point_t(-21.885963362223354, -0.8501530245969673),
                point_t(-22.266648787008304, -0.41191719217086753),
                point_t(-22.557674276002917, 0.02797728540450417),
                point_t(-22.757867972278643, 0.4677591083259667),
                point_t(-22.866423766477695, 0.9056574304108569),
                point_t(-22.88290454273092, 1.3399089896696292)},
            {point_t(-22.750448186516994, 2.577322687767994),
                point_t(-22.658195521928644, 3.2134285636369926),
                point_t(-22.476504380495015, 3.819167072106948),
                point_t(-22.206277598638863, 4.3898793380940395),
                point_t(-21.848742827627355, 4.921363665362133),
                point_t(-21.405445344948298, 5.40992308554063),
                point_t(-20.878240132700057, 5.8524052951093495),
                point_t(-20.269283282235513, 6.246234416464771),
                point_t(-19.581022777788117, 6.5894341483681735),
                point_t(-18.81618870479245, 6.8806420069493734),
                point_t(-17.97778292127308, 7.11911449882456),
                point_t(-17.069068223199757, 7.304723210536173),
                point_t(-16.09355702727847, 7.437941941154798),
                point_t(-15.054999587449595, 7.519825145216719),
                point_t(-13.957371754571646, 7.551978088963459),
                point_t(-12.804862282549474, 7.536519251932508),
                point_t(-11.601859678673303, 7.476035626261602),
                point_t(-10.35293859131028, 7.373531675694592),
                point_t(-9.062845724455034, 7.232372813469762),
                point_t(-7.736485266103889, 7.0562243414871695),
                point_t(-6.378903816050336, 6.848986861071072),
                point_t(-4.995274798565927, 6.614729217194662),
                point_t(-3.5908823465665125, 6.357620072408603),
                point_t(-2.171104646279044, 6.081859223380616),
                point_t(-0.7413967351040897, 5.791609771664),
                point_t(0.6927272497241084, 5.49093224111129),
                point_t(2.1257123680092045, 5.183721697566058),
                point_t(3.5519817081799325, 4.873648872714902),
                point_t(4.965955734663293, 4.564106224150109),
                point_t(6.362072123161831, 4.258159778926144),
                point_t(7.734806046307979, 3.958507509575999),
                point_t(9.078690861992644, 3.66744488128931),
                point_t(10.388339146135877, 3.386838088537986),
                point_t(11.658464000967767, 3.1181053708250106),
                point_t(12.883900559213151, 2.8622066625166545),
                point_t(14.059627594119481, 2.619641693084521),
                point_t(15.180789135231503, 2.3904565137805296),
                point_t(16.242715980394312, 2.1742582870699625),
                point_t(17.24094698584694, 1.9702380383195546),
                point_t(18.17125000863213, 1.7772009374964823),
                point_t(19.02964236906186, 1.593603554114358),
                point_t(19.812410695796554, 1.4175974133809905),
                point_t(20.51613001235287, 1.2470780773272496),
                point_t(21.137681921666694, 1.0797388833140675),
                point_t(21.67427174479717, 0.913128395205958),
                point_t(22.123444471033906, 0.744710560913827),
                point_t(22.48309937960664, 0.5719265249456934),
                point_t(22.751503197912854, 0.392257016792389),
                point_t(22.92730166766536, 0.20328422586863204),
                point_t(23.009529398582472, 0.002752081494166836),
                point_t(22.99761789913662, -0.21137611808811596),
                point_t(22.891401685354687, -0.4408637408098999),
                point_t(22.691122381611734, -0.6871514736456361),
                point_t(22.397430741644232, -0.9513134717194704),
                point_t(22.01138653347108, -1.2340197924577612),
                point_t(21.534456248374937, -1.5355058009053555),
                point_t(20.96850861136977, -1.8555491182663784),
                point_t(20.315807888456195, -2.193454562146787),
                point_t(19.57900500422757, -2.54804739586424),
                point_t(18.761126501811756, -2.9176750676700274),
                point_t(17.86556139548763, -3.3002174810065665),
                point_t(16.896045984371828, -3.693105696262873),
                point_t(15.856646713104212, -4.093348825165803),
                point_t(14.75174118224811, -4.4975687432144476),
                point_t(13.585997426953625, -4.902042115627187),
                point_t(12.364351597109447, -5.302749110234231),
                point_t(11.091984185549222, -5.6954280585930075),
                point_t(9.774294962715494, -6.075635226153649),
                point_t(8.416876786375516, -6.438808765191474),
                point_t(7.0254884634073616, -6.780335851874568),
                point_t(5.606026847233039, -7.095621952432097),
                point_t(4.164498359101356, -7.380161123860184),
                point_t(2.7069901240679335, -7.629606232605832),
                point_t(1.239640913173644, -7.839837970579132),
                point_t(-0.231387918008442, -8.007031561749164),
                point_t(-1.699941307560426, -8.127720084276623),
                point_t(-3.1598990003936103, -8.198853382138628),
                point_t(-4.605203521597402, -8.217851605742132),
                point_t(-6.02988744375544, -8.182652502068626),
                point_t(-7.428099792157041, -8.091751670162413),
                point_t(-8.794131443341216, -7.944235105755847),
                point_t(-10.122439385115896, -7.739803477801138),
                point_t(-11.40766971990917, -7.478787707755769),
                point_t(-12.644679307826337, -7.162155557606477),
                point_t(-13.82855596089965, -6.791509072659864),
                point_t(-14.954637115507854, -6.369072867841531),
                point_t(-16.018526925593896, -5.897673389351601),
                point_t(-17.016111734905166, -5.380709424736894),
                point_t(-17.94357390180843, -4.822114271503243),
                point_t(-18.79740396509073, -4.226310105114239),
                point_t(-19.574411153352113, -3.5981552095172136),
                point_t(-20.271732253951672, -2.94288484524705),
                point_t(-20.886838869820966, -2.2660466298960276),
                point_t(-21.41754310366757, -1.573431391708335),
                point_t(-21.86200171903512, -0.8710005278872294),
                point_t(-22.218718836266046, -0.16481095376034216),
                point_t(-22.486547228556102, 0.5390612336384389),
                point_t(-22.664688288946273, 1.2345972343378782),
                point_t(-22.75269074324532, 1.9159110216962458),
                point_t(-22.750448186516994, 2.577322687767991)},
            {point_t(-25.04492708422933, 2.0781951650376027),
                point_t(-24.88909123489873, 2.9231924487147927),
                point_t(-24.563186525114023, 3.7483122543890497),
                point_t(-24.073328045402963, 4.540966695303194),
                point_t(-23.428681508838004, 5.2889325067545325),
                point_t(-22.64116916017646, 5.980688515497077),
                point_t(-21.725086803616392, 6.605738363876396),
                point_t(-20.69664556371562, 7.1549077041470115),
                point_t(-19.573454683497747, 7.620605834823611),
                point_t(-18.373963765377727, 7.997042860737886),
                point_t(-17.116884298250298, 8.280394884336177),
                point_t(-15.820611035854444, 8.468911428951973),
                point_t(-14.50266377118476, 8.562961196998417),
                point_t(-13.179169289914661, 8.565014311372888),
                point_t(-11.86440180997657, 8.479561305753483),
                point_t(-10.570398077871925, 8.312971245107097),
                point_t(-9.306660572110026, 8.073293397761491),
                point_t(-8.079959058655415, 7.770008773517829),
                point_t(-6.894237168209136, 7.4137395222900135),
                point_t(-5.750626849744167, 7.0159255959013205),
                point_t(-4.647569636847667, 6.588479162741953),
                point_t(-3.581039784703899, 6.143427993086075),
                point_t(-2.544860636199489, 5.692559376723194),
                point_t(-1.5311021894026724, 5.247076082459197),
                point_t(-0.5305448880200163, 4.8172754232257295),
                point_t(0.4668077518931337, 4.412261667112111),
                point_t(1.4711868445423035, 4.039700862999069),
                point_t(2.492506416844817, 3.7056256712585656),
                point_t(3.53982287292476, 3.414296057559824),
                point_t(4.620825848396041, 3.1681197824236524),
                point_t(5.7413795356578845, 2.9676345687064103),
                point_t(6.905131743375254, 2.8115517258299167),
                point_t(8.113205509251069, 2.696858927134956),
                point_t(9.36398510325543, 2.61897784809462),
                point_t(10.653004846243068, 2.5719705475944554),
                point_t(11.972945449500475, 2.548786875366624),
                point_t(13.313738688467403, 2.5415438709896847),
                point_t(14.662777299183663, 2.541827128462362),
                point_t(16.005223170618777, 2.5410034682760028),
                point_t(17.324403337715797, 2.5305340062635304),
                point_t(18.602280087581754, 2.5022768417965984),
                point_t(19.819978790086928, 2.4487690998364284),
                point_t(20.95835495176759, 2.363478931115762),
                point_t(21.998580544689474, 2.241019268825685),
                point_t(22.922728932142483, 2.0773166136217585),
                point_t(23.714337727127617, 1.869729816757446),
                point_t(24.358929677141575, 1.6171156911018616),
                point_t(24.84447314250793, 1.3198402335824126),
                point_t(25.16176587234291, 0.9797362190432217),
                point_t(25.30472850514575, 0.6000098529940552),
                point_t(25.270597431791487, 0.18510097973512168),
                point_t(25.060010241600583, -0.2594970319896493),
                point_t(24.676980797971456, -0.7274502192314851),
                point_t(24.1287649208469, -1.211821170048589),
                point_t(23.425621547333105, -1.7053210165663493),
                point_t(22.580477958741962, -2.2005664965907648),
                point_t(21.60851106824614, -2.6903322591877985),
                point_t(20.526659735540267, -3.1677889105850574),
                point_t(19.353085506396596, -3.626717969863418),
                point_t(18.106600978344854, -4.061695902237016),
                point_t(16.806086104152516, -4.468240682462904),
                point_t(15.469913122633294, -4.8429158628531574),
                point_t(14.11540043833842, -5.183388821668295),
                point_t(12.758314671598015, -5.4884416840840355),
                point_t(11.412438308336235, -5.75793527108658),
                point_t(10.089217960224639, -5.992728271637262),
                point_t(8.797505287836886, -6.194555581323398),
                point_t(7.543399249745411, -6.365871341075848),
                point_t(6.330194641756511, -6.50966358290937),
                point_t(5.158438016643262, -6.629248494665412),
                point_t(4.026088166022394, -6.728053111039696),
                point_t(2.9287745438731956, -6.809395693903735),
                point_t(1.8601434531516012, -6.876273163838785),
                point_t(0.8122786316935395, -6.931164682896891),
                point_t(-0.2238198237180681, -6.975859875293726),
                point_t(-1.257716368145701, -7.01131923048811),
                point_t(-2.299000377370828, -7.03757299662701),
                point_t(-3.3567463342520925, -7.053663387326331),
                point_t(-4.438994187661727, -7.0576332462250075),
                point_t(-5.5522694946221085, -7.0465625040062445),
                point_t(-6.7011614987461385, -7.016651888965824),
                point_t(-7.887975180438347, -6.963351484626549),
                point_t(-9.112470613345893, -6.881529936255882),
                point_t(-10.371699775171471, -6.765678459783239),
                point_t(-11.659947404653751, -6.6101423638396115),
                point_t(-12.968778699148134, -6.409371613427382),
                point_t(-14.287192746391225, -6.1581810876807195),
                point_t(-15.60187672104314, -5.852010648845482),
                point_t(-16.89755219144141, -5.487174967188654),
                point_t(-18.157401508214363, -5.06109324603566),
                point_t(-19.36355930626804, -4.572489557955484),
                point_t(-20.49765175178265, -4.021555419246229),
                point_t(-21.54136439328787, -3.4100674644630167),
                point_t(-22.477018395002684, -2.7414545931646717),
                point_t(-23.288134580773654, -2.020810694477514),
                point_t(-23.95996511103184, -1.2548509502628329),
                point_t(-24.479973739206194, -0.4518117072311505),
                point_t(-24.838247407563664, 0.3787040788784849),
                point_t(-25.027824380130127, 1.225931860293217),
                point_t(-25.04492708422933, 2.078195165037599)},
            {point_t(-24.857777819857336, 2.5196823412543),
                point_t(-24.73430476189517, 3.368247847238699),
                point_t(-24.45068505841098, 4.168406668273659),
                point_t(-24.01032322120573, 4.909170981546228),
                point_t(-23.41921210001421, 5.581643814304959),
                point_t(-22.68584218042573, 6.179143289308971),
                point_t(-21.821038596315656, 6.6972151375202325),
                point_t(-20.837725369746156, 7.1335425788480755),
                point_t(-19.7506189208795, 7.4877683706032245),
                point_t(-18.575855746362276, 7.761248284059005),
                point_t(-17.33056222250121, 7.956758239971311),
                point_t(-16.032377593461376, 8.078178666356362),
                point_t(-14.698944175803458, 8.130179301068416),
                point_t(-13.34738145782817, 8.117925725661907),
                point_t(-11.993762904339766, 8.046825569787327),
                point_t(-10.65261571655855, 7.922327843465603),
                point_t(-9.336464390944016, 7.74978358707177),
                point_t(-8.055438555442818, 7.534370372867129),
                point_t(-6.8169641708907065, 7.281077565161816),
                point_t(-5.62555475814201, 6.994744058298145),
                point_t(-4.482715896871627, 6.680135836237007),
                point_t(-3.386971949245776, 6.342047446856943),
                point_t(-2.3340189575465455, 5.985409587927274),
                point_t(-1.317002166756106, 5.6153845919656025),
                point_t(-0.32691088889361086, 5.237432699001555),
                point_t(0.646922259275669, 4.857334536544589),
                point_t(1.6162360524630373, 4.481158999211077),
                point_t(2.59319229435317, 4.115170461160625),
                point_t(3.589691193770352, 3.7656746164465),
                point_t(4.616679923694304, 3.438807832067356),
                point_t(5.683485129950024, 3.1402803020920045),
                point_t(6.797200038430917, 2.875088103050628),
                point_t(7.962155207678199, 2.647213101393795),
                point_t(9.179498921693792, 2.4593322457620945),
                point_t(10.446908828079616, 2.312558866412803),
                point_t(11.75845088477357, 2.2062380778454975),
                point_t(13.104595235862883, 2.1378162247204218),
                point_t(14.472391597202378, 2.102800624135207),
                point_t(15.84579943722886, 2.0948208450611627),
                point_t(17.206161048284024, 2.105796729137837),
                point_t(18.53279887996044, 2.1262116733545606),
                point_t(19.803712590141693, 2.1454827940943764),
                point_t(20.99634646516162, 2.152412927969164),
                point_t(22.088394416742375, 2.135703447258401),
                point_t(23.05860786039042, 2.08450199132327),
                point_t(23.88757151936416, 1.9889557923551706),
                point_t(24.558413597360286, 1.8407395699054128),
                point_t(25.057419753321703, 1.6335271441629153),
                point_t(25.374524742932095, 1.3633780157225344),
                point_t(25.503661239326686, 1.0290141002077648),
                point_t(25.44295192463363, 0.6319673910238721),
                point_t(25.19473812247241, 0.17658624580583288),
                point_t(24.765445659792846, -0.33010414563462837),
                point_t(24.16529593621352, -0.8786832159580341),
                point_t(23.407876983281334, -1.4577461900676234),
                point_t(22.50959528791383, -2.054424987065579),
                point_t(21.48903405393872, -2.6549921169565125),
                point_t(20.366247164677237, -3.24551553055037),
                point_t(19.162020241394973, -3.812528873813408),
                point_t(17.897130798927343, -4.343680128639058),
                point_t(16.591638592857255, -4.828322310033137),
                point_t(15.264234921898947, -5.2580127240577035),
                point_t(13.931676055020981, -5.626892129611349),
                point_t(12.60832131583844, -5.931921724856425),
                point_t(11.305790943802549, -6.1729638222652),
                point_t(10.032752959968764, -6.352700925176387),
                point_t(8.794842205026534, -6.476397155207569),
                point_t(7.594708794905303, -6.551515058418102),
                point_t(6.432187739780374, -6.587209199272851),
                point_t(5.304576645050548, -6.593725134639184),
                point_t(4.207004459179886, -6.581737914262966),
                point_t(3.132871297512383, -6.561667844500789),
                point_t(2.0743375352826363, -6.543012660356927),
                point_t(1.022839645345392, -6.533734389155916),
                point_t(-0.030389386391081047, -6.539736105331287),
                point_t(-1.0938159005551806, -6.5644586506626315),
                point_t(-2.175136325172815, -6.608620530694861),
                point_t(-3.2808587048930993, -6.670116002432231),
                point_t(-4.41594757527957, -6.744077325892732),
                point_t(-5.583541246927312, -6.823097797098814),
                point_t(-6.784747015196716, -6.897603063580171),
                point_t(-8.018516412340905, -6.956349879664369),
                point_t(-9.281599528692839, -6.987024372779871),
                point_t(-10.568574771226213, -6.97690647042129),
                point_t(-11.871948282844006, -6.9135636845068955),
                point_t(-13.182315652571681, -6.7855361494210635),
                point_t(-14.488577501733884, -6.582975715266327),
                point_t(-15.77819999244949, -6.2982049291150775),
                point_t(-17.03751119926622, -5.9261666879179185),
                point_t(-18.252024516553508, -5.464741897773353),
                point_t(-19.406780734531416, -4.914920213765998),
                point_t(-20.486700994163563, -4.280817384728716),
                point_t(-21.47694342179109, -3.569541374272983),
                point_t(-22.363256761215773, -2.790917755496498),
                point_t(-23.132324700884897, -1.9570923921596435),
                point_t(-23.772094803012664, -1.0820356919634828),
                point_t(-24.272085977738758, -0.1809773998443559),
                point_t(-24.623668338085643, 0.7301962528363947),
                point_t(-24.82030907830138, 1.6355503395735032),
                point_t(-24.857777819857336, 2.5196823412542964)},
            {point_t(-25.490714588138275, 2.3978243413089206),
                point_t(-25.302766934147513, 3.458419438973317),
                point_t(-24.8979072729005, 4.461603518114615),
                point_t(-24.29164686735067, 5.376115107504654),
                point_t(-23.50654482740445, 6.175707219807783),
                point_t(-22.57046313813481, 6.841004130700666),
                point_t(-21.51446927757899, 7.360781470757349),
                point_t(-20.370579101402626, 7.732552153284076),
                point_t(-19.169543884988865, 7.962405429209987),
                point_t(-17.93887679702586, 8.064116631206607),
                point_t(-16.70128665364484, 8.057614119570298),
                point_t(-15.473643456656909, 7.966950835731471),
                point_t(-14.266545412912144, 7.817974722738049),
                point_t(-13.084496442692732, 7.635920450890453),
                point_t(-11.926642746896938, 7.443151567799399),
                point_t(-10.787962885684678, 7.257266650597418),
                point_t(-9.660763420896307, 7.0897467273108905),
                point_t(-8.536305670054848, 6.94526765152883),
                point_t(-7.406381055067044, 6.821735463070475),
                point_t(-6.264663559459576, 6.711031450917652),
                point_t(-5.107696639147002, 6.6003836303059815),
                point_t(-3.935415452688471, 6.474219519375047),
                point_t(-2.751158879938294, 6.316307531201121),
                point_t(-1.56118384795561, 6.111965677096229),
                point_t(-0.3737508920800146, 5.8501094489258385),
                point_t(0.8021012721105808, 5.52492642064287),
                point_t(1.9579382519974442, 5.137001784326868),
                point_t(3.087296062810644, 4.6937731528969335),
                point_t(4.186856433392757, 4.209259243462377),
                point_t(5.257275048605025, 3.703079025134397),
                point_t(6.303541743014279, 3.198848530644),
                point_t(7.334800672393437, 2.7221048747125907),
                point_t(8.36361626502797, 2.2979549793563256),
                point_t(9.40473139870279, 1.9486753653316469),
                point_t(10.473421513096843, 1.691496282104953),
                point_t(11.583596280341835, 1.536787710955067),
                point_t(12.745833931896438, 1.486827892340917),
                point_t(13.965548695510913, 1.5352805658654176),
                point_t(15.241487136937158, 1.6674403452390962),
                point_t(16.564724624770008, 1.8612330490743578),
                point_t(17.91829074055671, 2.0888863954231303),
                point_t(19.277496143927294, 2.3191231371092442),
                point_t(20.61096851010818, 2.519679555446707),
                point_t(21.882337933936093, 2.659921934796607),
                point_t(23.052449189568797, 2.7133250526375092),
                point_t(24.08192565162784, 2.6595905622651697),
                point_t(24.93387276353657, 2.486217959479842),
                point_t(25.57649144040839, 2.1893931243741234),
                point_t(25.985375612632062, 1.7741240524149626),
                point_t(26.145293089337756, 1.2536240123726337),
                point_t(26.051292832770073, 0.6480121349437898),
                point_t(25.709040525582807, -0.017536370779311333),
                point_t(25.134352469616303, -0.7150094721902347),
                point_t(24.351968943405563, -1.4159354670231452),
                point_t(23.393675438647886, -2.093696556868557),
                point_t(22.29593732595469, -2.725568887629122),
                point_t(21.097255081106628, -3.2943106217024254),
                point_t(19.835469353384404, -3.7891677227796654),
                point_t(18.54524587733292, -4.2062268609347),
                point_t(17.25594959130338, -4.548110109763134),
                point_t(15.990077463715734, -4.823070364244468),
                point_t(14.762364420816931, -5.043603298922116),
                point_t(13.579611848314103, -5.224735671014495),
                point_t(12.44121974815529, -5.382176677740306),
                point_t(11.340338404139438, -5.530526498089737),
                point_t(10.265499616166673, -5.681723678976734),
                point_t(9.202546496298474, -5.843882274831742),
                point_t(8.136658282140418, -6.02062406584487),
                point_t(7.05426455870163, -6.2109556822233785),
                point_t(5.944661595161931, -6.409680928218107),
                point_t(4.801180088261589, -6.6082812614960025),
                point_t(3.6218045555131657, -6.796148186070417),
                point_t(2.409204640059984, -6.962015313530565),
                point_t(1.1702015164055874, -7.095418685660487),
                point_t(-0.08524797606731524, -7.188013506952917),
                point_t(-1.3454168948381593, -7.234593625287776),
                point_t(-2.598355411222905, -7.233694897832053),
                point_t(-3.8334341708759423, -7.187711233121579),
                point_t(-5.042700470434567, -7.102507517472364),
                point_t(-6.221885428239032, -6.986570911901788),
                point_t(-7.370938859726229, -6.849795041803451),
                point_t(-8.494020112914743, -6.702034723488611),
                point_t(-9.598932348751614, -6.55159742518386),
                point_t(-10.69604856475063, -6.403848493002909),
                point_t(-11.79683380177807, -6.260098965830411),
                point_t(-12.912113661077575, -6.116918215484965),
                point_t(-14.05026976665055, -5.965971258270192),
                point_t(-15.215554944373347, -5.794426634832248),
                point_t(-16.406713314915333, -5.585920730628223),
                point_t(-17.616063862521912, -5.3220044781580675),
                point_t(-18.82916292005218, -4.983944771460986),
                point_t(-20.02510564293642, -4.554711281705706),
                point_t(-21.177464413701536, -4.020954178264699),
                point_t(-22.25579940053173, -3.374772393021476),
                point_t(-23.22761943942529, -2.6150864443111077),
                point_t(-24.060625721216507, -1.7484633436273764),
                point_t(-24.725041023119864, -0.7892906755124692),
                point_t(-25.195816432385055, 0.24074215699896806),
                point_t(-25.45451679184906, 1.3138312154113212),
                point_t(-25.490714588138275, 2.397824341308916)},
            {point_t(-25.39003468512172, 2.4005111080777786),
                point_t(-25.22722396258436, 3.4736559398983196),
                point_t(-24.858323831189107, 4.487206909086042),
                point_t(-24.293693842987288, 5.4084173408883425),
                point_t(-23.549928962197242, 6.210080544112792),
                point_t(-22.648969041584742, 6.872524081388842),
                point_t(-21.616849869416942, 7.384932369390641),
                point_t(-20.482166914803823, 7.74587404228524),
                point_t(-19.274352388505015, 7.962989759688456),
                point_t(-18.021890691168252, 8.051879689562432),
                point_t(-16.75061302476187, 8.034309020650545),
                point_t(-15.482215604908715, 7.935916364349916),
                point_t(-14.23313525690319, 7.78365700556032),
                point_t(-13.01389045948937, 7.60323598594777),
                point_t(-11.828956241535746, 7.41678286665598),
                point_t(-10.677190830756574, 7.240991402386432),
                point_t(-9.552775478918406, 7.085896591372395),
                point_t(-8.446572640013583, 6.954394214253713),
                point_t(-7.34775846179959, 6.842531215353852),
                point_t(-6.245549918021345, 6.740517107318436),
                point_t(-5.130830248825613, 6.63433495379544),
                point_t(-3.9974820969277682, 6.5077725029964),
                point_t(-2.8432666347030433, 6.344655235813819),
                point_t(-1.670136975527716, 6.131046893348956),
                point_t(-0.48394031699477047, 5.857190522897094),
                point_t(0.7064617423860038, 5.5189928884026),
                point_t(1.8905503318621304, 5.118903708232846),
                point_t(3.057814024282674, 4.666103338147253),
                point_t(4.199503995189447, 4.175981782104202),
                point_t(5.3102402687387675, 3.668961383849856),
                point_t(6.389236605092197, 3.1687785437647484),
                point_t(7.440948176214443, 2.7003904928673084),
                point_t(8.475009280384297, 2.287707094654725),
                point_t(9.505411301719347, 1.951362132100505),
                point_t(10.548964484659994, 1.706732783029956),
                point_t(11.623179722053228, 1.5623911019264936),
                point_t(12.743786956259823, 1.5191301257246055),
                point_t(13.922164560718123, 1.5696538901704262),
                point_t(15.162981233487226, 1.6989602959272718),
                point_t(16.46234403293205, 1.8853839477076495),
                point_t(17.806702927155513, 2.102208284424294),
                point_t(19.172687640411144, 2.3197074675877123),
                point_t(20.527954615965786, 2.5074426138025308),
                point_t(21.833011562819063, 2.6366168358768545),
                point_t(23.04387704131699, 2.682290581255954),
                point_t(24.115335807636797, 2.6252728450874403),
                point_t(25.004478746739935, 2.45353349453716),
                point_t(25.674177945769582, 2.1630244232307043),
                point_t(26.096147667560167, 1.757848804203976),
                point_t(26.25328103131566, 1.2497738764341382),
                point_t(26.141025862811336, 0.6571386976686726),
                point_t(25.767663118850262, 0.003259381504066086),
                point_t(25.153466111054534, -0.6855238157894508),
                point_t(24.32883533372695, -1.3819841435336868),
                point_t(23.331608794408588, -2.060143573247204),
                point_t(22.20382957118994, -2.697221183016425),
                point_t(20.98830195353452, -3.275229405449699),
                point_t(19.72527992846965, -3.7820866488084097),
                point_t(18.449606347608345, -4.21216039317497),
                point_t(17.188561671168067, -4.566208185857156),
                point_t(15.960595425187764, -4.850740178994148),
                point_t(14.775011982613622, -5.07688076028029),
                point_t(13.632577068447846, -5.258853312299037),
                point_t(12.52691461023321, -5.412246664619558),
                point_t(11.446485907960444, -5.5522408799350185),
                point_t(10.376892631523, -5.691971563678335),
                point_t(9.303226399315031, -5.841195508062884),
                point_t(8.212201253703569, -6.005387564919867),
                point_t(7.093848000413022, -6.185352291251951),
                point_t(5.942614619525316, -6.377378694834418),
                point_t(4.757795953468798, -6.573907937190993),
                point_t(3.5432986520632337, -6.764628235382242),
                point_t(2.3068240482220292, -6.937864414897273),
                point_t(1.058613703004391, -7.082096796659323),
                point_t(-0.1900564795834646, -7.187429176474449),
                point_t(-1.4284307889805534, -7.246830566931952),
                point_t(-2.6476817823399346, -7.256999996751806),
                point_t(-3.8420063191277496, -7.218745704503134),
                point_t(-5.009290314425612, -7.136825234650093),
                point_t(-6.151279445035669, -7.0192553768444705),
                point_t(-7.273252354365036, -6.876163742946869),
                point_t(-8.38324805798664, -6.718309971699598),
                point_t(-9.490944406773712, -6.555447561122355),
                point_t(-10.606315534709365, -6.394721930278026),
                point_t(-11.738211208510615, -6.239303213547034),
                point_t(-12.893000019639343, -6.087432559084181),
                point_t(-14.07340337632916, -5.932019934780733),
                point_t(-15.277621588612645, -5.760873651210894),
                point_t(-16.498821069680083, -5.557573026015525),
                point_t(-17.72501699009402, -5.302923261905342),
                point_t(-18.939352344966935, -4.976863697489731),
                point_t(-20.120745172660996, -4.560644813945975),
                point_t(-21.24485233383685, -4.039052254358721),
                point_t(-22.2852814390597, -3.402442207771156),
                point_t(-23.2149718776286, -2.648363905669282),
                point_t(-24.007660501082764, -1.7825809849119174),
                point_t(-24.639346161041946, -0.8193606623917212),
                point_t(-25.08966892856405, 0.219027775153686),
                point_t(-25.34312377649273, 1.3035833307097207),
                point_t(-25.39003468512172, 2.400511108077774)},
            {point_t(-25.655944840882146, 2.2830512599591044),
                point_t(-25.44626653916609, 3.4461856056686533),
                point_t(-24.98797238624933, 4.555059369621578),
                point_t(-24.308777508434332, 5.558419249729171),
                point_t(-23.44751929079057, 6.413109492149325),
                point_t(-22.448948555957102, 7.089162622993216),
                point_t(-21.35805193147527, 7.573120798214706),
                point_t(-20.21483634798868, 7.869076153848289),
                point_t(-19.050390599901494, 7.997286243070843),
                point_t(-17.88477917299888, 7.99061198803111),
                point_t(-16.7269715564456, 7.889372069963248),
                point_t(-15.576634104784446, 7.735449227573439),
                point_t(-14.427282663705476, 7.566579760547986),
                point_t(-13.270073651496267, 7.411693511964682),
                point_t(-12.097438040185466, 7.287962545731503),
                point_t(-10.905846320845193, 7.1999033035029),
                point_t(-9.69721190052058, 7.140517842517501),
                point_t(-8.478748106244481, 7.094120267563915),
                point_t(-7.261426204880661, 7.040234687456411),
                point_t(-6.057471080025887, 6.957814477353851),
                point_t(-4.877519757512124, 6.8290386059026105),
                point_t(-3.7281194048564825, 6.642081322138766),
                point_t(-2.6101476798255505, 6.392493577965808),
                point_t(-1.5185210857259421, 6.083127091559923),
                point_t(-0.44326325081633994, 5.7228160592556),
                point_t(0.6283026576885217, 5.324252151611763),
                point_t(1.708729442120603, 4.901605013206465),
                point_t(2.807631295855135, 4.46843455798935),
                point_t(3.929531635094615, 4.036319702339954),
                point_t(5.072892580099315, 3.614420967584874),
                point_t(6.230593911439925, 3.209948645019294),
                point_t(7.391810468446074, 2.829278063145643),
                point_t(8.544916493287303, 2.47928900871936),
                point_t(9.68079116498327, 2.168443350751582),
                point_t(10.795767537401138, 1.9071676555326371),
                point_t(11.893489910889569, 1.7072658144369488),
                point_t(12.985124389657878, 1.5803176985265945),
                point_t(14.08767431847595, 1.5352749442524913),
                point_t(15.220530105339035, 1.5756893978700353),
                point_t(16.400759082461132, 1.6971537631169773),
                point_t(17.637940683025185, 1.885563124534516),
                point_t(18.929512773109906, 2.1167083236667263),
                point_t(20.257578777834883, 2.357501145100862),
                point_t(21.58792739100054, 2.568843717237022),
                point_t(22.87166687375001, 2.7098437551529173),
                point_t(24.049433718537294, 2.7428029475784617),
                point_t(25.057679423953974, 2.638222417813576),
                point_t(25.836152641491836, 2.3790153810473984),
                point_t(26.335449492943177, 1.9632078166128386),
                point_t(26.523450273190907, 1.4046311229780315),
                point_t(26.3896099940642, 0.7314291203607606),
                point_t(25.94640029518529, -0.017440276661394152),
                point_t(25.22765505958429, -0.7971947798636488),
                point_t(24.284072498342116, -1.562945839405517),
                point_t(23.17658474188299, -2.275262844405062),
                point_t(21.96864174437857, -2.9047332847952068),
                point_t(20.71861134544023, -3.434846501715233),
                point_t(19.47344614093104, -3.862819571394431),
                point_t(18.26452213569094, -4.198335093542151),
                point_t(17.1061605651905, -4.460508803394662),
                point_t(15.99687534636254, -4.673687923615181),
                point_t(14.922929295862495, -4.862849782652444),
                point_t(13.863414082481993, -5.04939706824678),
                point_t(12.795855028964883, -5.2480304395181605),
                point_t(11.701315776507785, -5.465146748664671),
                point_t(10.5681375149103, -5.698908583873702),
                point_t(9.393756691811532, -5.940816878595287),
                point_t(8.184440777544154, -6.178352103192882),
                point_t(6.953186366636902, -6.398079464297943),
                point_t(5.716360851574306, -6.588568176477235),
                point_t(4.489876524304298, -6.742557939309592),
                point_t(3.2857292945837857, -6.857995878929379),
                point_t(2.1096110607512735, -6.937822659130667),
                point_t(0.9600453803195754, -6.988653748332594),
                point_t(-0.1708434008857478, -7.0187265159358505),
                point_t(-1.295166469019022, -7.035621396698962),
                point_t(-2.426239078837679, -7.044289927424677),
                point_t(-3.5753776516850366, -7.045831741623621),
                point_t(-4.74924081852382, -7.037278092128782),
                point_t(-5.948296930242811, -7.012401826137799),
                point_t(-7.166745251437571, -6.963334379839086),
                point_t(-8.393894393281032, -6.882580885224929),
                point_t(-9.616677227046786, -6.764926058811354),
                point_t(-10.822724199731335, -6.608738406280315),
                point_t(-12.00328064176457, -6.416307027484133),
                point_t(-13.155267806164558, -6.193058965045399),
                point_t(-14.281951032257817, -5.945761891016073),
                point_t(-15.391960228158332, -5.680063199195401),
                point_t(-16.496752197746204, -5.397899266388732),
                point_t(-17.606942271801504, -5.095386363850774),
                point_t(-18.72819562360675, -4.761756311262216),
                point_t(-19.857501876046108, -4.379729376787956),
                point_t(-20.980630338117756, -3.927452941794834),
                point_t(-22.071378631806937, -3.3818256829922198),
                point_t(-23.09291683078264, -2.722732803532881),
                point_t(-24.00114982647746, -1.9374968126991936),
                point_t(-24.749643886121348, -1.024746988747665),
                point_t(-25.29536108934302, 0.0030460736050042236),
                point_t(-25.604275872783035, 1.1189384368404534),
                point_t(-25.655944840882146, 2.2830512599591)},
            {point_t(-25.548591738649275, 2.3238940617230646),
                point_t(-25.38222647539509, 3.4550064931304423),
                point_t(-24.983402748814605, 4.5296328218849915),
                point_t(-24.364831222553796, 5.50516041646608),
                point_t(-23.550053934556775, 6.345455643345081),
                point_t(-22.57209451100398, 7.024182908988381),
                point_t(-21.47073931992182, 7.527209681808356),
                point_t(-20.28863400133708, 7.853817066908589),
                point_t(-19.066679278270367, 8.016529072243372),
                point_t(-17.83944922254234, 8.03950174354176),
                point_t(-16.631459780138623, 7.955573805586869),
                point_t(-15.455038242783624, 7.802260162934069),
                point_t(-14.31028150348509, 7.617143413417039),
                point_t(-13.187186731547037, 7.43325260972804),
                point_t(-12.069577833676115, 7.275077701163845),
                point_t(-10.940041992587961, 7.155825378941364),
                point_t(-9.78483583993173, 7.076367753243904),
                point_t(-8.597692684243384, 7.026083161024044),
                point_t(-7.381681556249848, 6.985476410395014),
                point_t(-6.1486966297935295, 6.930150611294689),
                point_t(-4.916699185817542, 6.835448778717727),
                point_t(-3.7053676931261212, 6.680948239240725),
                point_t(-2.5312051132744364, 6.454011067747032),
                point_t(-1.4033049722463415, 6.151774214706403),
                point_t(-0.32084275938235274, 5.781273061072745),
                point_t(0.7270406845692545, 5.357770257711841),
                point_t(1.758873277252272, 4.901727563661545),
                point_t(2.7965295871617997, 4.435130633166934),
                point_t(3.859985357510784, 3.9779919239647796),
                point_t(4.962448346104344, 3.545785506809269),
                point_t(6.107116938868846, 3.1483223141071486),
                point_t(7.286454151313644, 2.790209262163898),
                point_t(8.484262361665301, 2.472634844829983),
                point_t(9.680142331846511, 2.195882677794023),
                point_t(10.85528770415838, 1.9617774971085156),
                point_t(11.998162065314139, 1.77526803948043),
                point_t(13.108539616781563, 1.644555260937927),
                point_t(14.198694802737295, 1.5795406270042587),
                point_t(15.291145240242876, 1.5888149140551042),
                point_t(16.413152581494625, 1.6758275286492934),
                point_t(17.588985650941613, 1.8351657749397876),
                point_t(18.83156062549509, 2.0499551730357672),
                point_t(20.135342963566746, 2.291234096502972),
                point_t(21.472248158307696, 2.5197820295929425),
                point_t(22.79173023716104, 2.6903657533792646),
                point_t(24.025407791053492, 2.7578229557896003),
                point_t(25.095625977012453, 2.6839508692250016),
                point_t(25.926497712479833, 2.44391493039674),
                point_t(26.455398907893073, 2.0309042267434774),
                point_t(26.64274064527061, 1.458044505273728),
                point_t(26.47814421377885, 0.7570832119226916),
                point_t(25.981841068599596, -0.02601803509572168),
                point_t(25.20106063541157, -0.837840209154564),
                point_t(24.202152682934084, -1.6254040471043898),
                point_t(23.060008006727752, -2.3437755468952575),
                point_t(21.84682054499413, -2.9620146418811837),
                point_t(20.6222813222035, -3.466444355639623),
                point_t(19.426911478718406, -3.8607617442455946),
                point_t(18.279523587754344, -4.163140777673505),
                point_t(17.178913246484633, -4.401057568547071),
                point_t(16.109023664757203, -4.604979390408634),
                point_t(15.046178090905563, -4.802219199616195),
                point_t(13.966667529633304, -5.012141607018118),
                point_t(12.853062154868775, -5.243549699380184),
                point_t(11.698043152026614, -5.494571225936567),
                point_t(10.505210827277896, -5.754814446280978),
                point_t(9.287052422715423, -6.009099007401687),
                point_t(8.060880547015914, -6.241782832230549),
                point_t(6.843944574777608, -6.440655141604838),
                point_t(5.648999338570084, -6.599546905625477),
                point_t(4.481390683809157, -6.719169773257114),
                point_t(3.3382601147268223, -6.806141681109613),
                point_t(2.2099049501643293, -6.870585308256633),
                point_t(1.0827980657515486, -6.922997311798166),
                point_t(-0.056602574902057695, -6.97121619447742),
                point_t(-1.2182606052074245, -7.018244103611721),
                point_t(-2.406071622451812, -7.061429975404218),
                point_t(-3.617036877096891, -7.093164675210598),
                point_t(-4.842216051260404, -7.102861753208972),
                point_t(-6.069130403250521, -7.079689375312582),
                point_t(-7.284950528934919, -7.015349084620769),
                point_t(-8.479648136488159, -6.90619937079403),
                point_t(-9.648343659715339, -6.754189351833453),
                point_t(-10.792313841447081, -6.566355391302375),
                point_t(-11.91846606380969, -6.352970991988409),
                point_t(-13.037447832224194, -6.124749669695322),
                point_t(-14.160851788544367, -5.889713856132317),
                point_t(-15.298135204733455, -5.6504174138071654),
                point_t(-16.45387356491288, -5.4021353990839796),
                point_t(-17.625828362044373, -5.132435633072864),
                point_t(-18.804081452828104, -4.822271140228197),
                point_t(-19.971241354990248, -4.448441798756679),
                point_t(-21.103526854543556, -3.9870267270975037),
                point_t(-22.172425241508265, -3.4172302913763506),
                point_t(-23.146619348241877, -2.7250356081939553),
                point_t(-23.993959039633797, -1.9061168131522503),
                point_t(-24.68337403945416, -0.9676013979734971),
                point_t(-25.18673214772942, 0.07153935185864424),
                point_t(-25.480695053528628, 1.1814984631371068),
                point_t(-25.548591738649275, 2.32389406172306)},
            {point_t(-25.672207822734908, 2.338034296266844),
                point_t(-25.45545172640074, 3.5191188644544367),
                point_t(-24.982988666930876, 4.623362105090202),
                point_t(-24.29090927585301, 5.598748226394321),
                point_t(-23.426093818808916, 6.409188511604001),
                point_t(-22.437452687098666, 7.037826100220703),
                point_t(-21.368163615615323, 7.486431579195999),
                point_t(-20.25069147798142, 7.771564429906673),
                point_t(-19.105416418941907, 7.918916531930643),
                point_t(-17.94256725878142, 7.9575205917702565),
                point_t(-16.766219464036567, 7.915252478792907),
                point_t(-15.578654326869257, 7.816400397477849),
                point_t(-14.383506754490739, 7.681255784741033),
                point_t(-13.186772649663308, 7.52698189293325),
                point_t(-11.99565588697533, 7.368665511092085),
                point_t(-10.816081876840101, 7.219558247200284),
                point_t(-9.650194016026418, 7.090010944476226),
                point_t(-8.495116979936888, 6.985305058411687),
                point_t(-7.343739032894188, 6.903223773393098),
                point_t(-6.187433770465068, 6.832538070981959),
                point_t(-5.019817222056623, 6.753467626946224),
                point_t(-3.8401273770240634, 6.640626912446763),
                point_t(-2.65482119736007, 6.468151302290812),
                point_t(-1.4765302232519906, 6.215886586030398),
                point_t(-0.32042867749862364, 5.875002344277955),
                point_t(0.8009626312700405, 5.451358067640081),
                point_t(1.882833393000132, 4.965460431920466),
                point_t(2.9311714110671123, 4.448773824399256),
                point_t(3.96256106181728, 3.937213821352423),
                point_t(5.000390869460004, 3.463532869807353),
                point_t(6.0683797981973076, 3.0507097737944195),
                point_t(7.183336115074564, 2.7082281103923953),
                point_t(8.34950267776736, 2.4323135180360205),
                point_t(9.556526247760878, 2.2100229123378026),
                point_t(10.782062453152731, 2.02588986843251),
                point_t(11.998576147197868, 1.8689973226856404),
                point_t(13.182461563482349, 1.7381430708661676),
                point_t(14.322654918485155, 1.643273495263179),
                point_t(15.425787064148189, 1.6024581052874258),
                point_t(16.51572828580112, 1.6350494260369368),
                point_t(17.626928174297273, 1.7529131379378713),
                point_t(18.79282348482355, 1.9523426327230382),
                point_t(20.032224927327665, 2.2092529447314693),
                point_t(21.337488474409753, 2.47946070279898),
                point_t(22.668114153075408, 2.704505987923044),
                point_t(23.952182540047843, 2.821935327113595),
                point_t(25.096040058896183, 2.777680152430212),
                point_t(26.00041965918062, 2.5375027403249804),
                point_t(26.579359023640933, 2.0946370950023976),
                point_t(26.777382469175922, 1.4716876965060497),
                point_t(26.580719918085347, 0.7163051093103351),
                point_t(26.019783591955257, -0.10827067209763809),
                point_t(25.16232349474003, -0.9354527494672931),
                point_t(24.099034646695003, -1.7073851988758924),
                point_t(22.92524832282981, -2.3840968736892196),
                point_t(21.723204460908498, -2.9478744073374044),
                point_t(20.54905607119785, -3.4023319843156283),
                point_t(19.427325560602135, -3.7670324610403845),
                point_t(18.35344553445513, -4.069552967745265),
                point_t(17.302873362232493, -4.337324700288152),
                point_t(16.243665488662515, -4.591336199176312),
                point_t(15.148753795212059, -4.842997302228551),
                point_t(14.004610052988964, -5.094394244020035),
                point_t(12.814325014197236, -5.341162239692912),
                point_t(11.594925115787532, -5.57655237770807),
                point_t(10.370451143379954, -5.79513577307494),
                point_t(9.16343633862979, -5.994958772857907),
                point_t(7.987655296010265, -6.177670460906555),
                point_t(6.8443586566613375, -6.3469258583996275),
                point_t(5.7229212852708695, -6.505959095697237),
                point_t(4.6053507995570175, -6.6554369049981945),
                point_t(3.472901938632135, -6.792498489877292),
                point_t(2.312480654470826, -6.911363410868989),
                point_t(1.1207405891072089, -7.005249948800082),
                point_t(-0.09533971557359651, -7.068828734790149),
                point_t(-1.3213786414465056, -7.100225255383223),
                point_t(-2.5408313063497543, -7.10175130219818),
                point_t(-3.7406529611825245, -7.079024440666819),
                point_t(-4.9154413022660535, -7.038749381884978),
                point_t(-6.068716321366793, -6.985960092107372),
                point_t(-7.211028582234134, -6.921761274692528),
                point_t(-8.355688020740299, -6.84246650253511),
                point_t(-9.513701835810027, -6.7405461606011325),
                point_t(-10.689738137140585, -6.6071334939147315),
                point_t(-11.88052354045403, -6.435223628990325),
                point_t(-13.076184972895733, -6.2223622100080505),
                point_t(-14.263969824783448, -5.971695007903819),
                point_t(-15.432894888631397, -5.690738740601128),
                point_t(-16.57748964899851, -5.3879951645402),
                point_t(-17.699053613050022, -5.068323261748869),
                point_t(-18.803667370944375, -4.728541857022987),
                point_t(-19.897319408289462, -4.354853988828439),
                point_t(-20.979566738795697, -3.9232938588385835),
                point_t(-22.037783417602952, -3.403587100144029),
                point_t(-23.04404364393538, -2.765813710806312),
                point_t(-23.956016516278137, -1.9883694501541664),
                point_t(-24.7221111801257, -1.065213938286226),
                point_t(-25.2898501839685, -0.010441799912858027),
                point_t(-25.61545473742657, 1.1411771363431444),
                point_t(-25.672207822734908, 2.3380342962668395)},
            {point_t(-25.607570286934994, 2.311891243134241),
                point_t(-25.429309873921216, 3.4557623160484052),
                point_t(-25.005523690173742, 4.547466870120397),
                point_t(-24.353344696273037, 5.539872423981323),
                point_t(-23.504113569727437, 6.39026188143088),
                point_t(-22.500671236507976, 7.066219797926292),
                point_t(-21.39195989982395, 7.5510874098963985),
                point_t(-20.225797808398035, 7.847301567079073),
                point_t(-19.041527871300673, 7.976238429141212),
                point_t(-17.86456583394012, 7.974102692455704),
                point_t(-16.704483556738314, 7.88463672656109),
                point_t(-15.557227571772602, 7.750510387423956),
                point_t(-14.410734004599517, 7.605753006243207),
                point_t(-13.252049994173877, 7.471271619216632),
                point_t(-12.073560444916493, 7.354444636664098),
                point_t(-10.87627297920178, 7.252365226491019),
                point_t(-9.669229667084988, 7.157068788228744),
                point_t(-8.465583564835622, 7.060497453337089),
                point_t(-7.277138620266882, 6.95726632809249),
                point_t(-6.109704522710203, 6.8443833999373895),
                point_t(-4.961231531969418, 6.718502454443791),
                point_t(-3.8235019972906112, 6.572468756560028),
                point_t(-2.6866310398082582, 6.393345003305739),
                point_t(-1.5443866430259923, 6.163566166619823),
                point_t(-0.3979043483035274, 5.865544488037255),
                point_t(0.7440413442811293, 5.4884462263893745),
                point_t(1.8686350249399164, 5.03465027045596),
                point_t(2.965225651047282, 4.5231187038419804),
                point_t(4.031605163075207, 3.987759423246846),
                point_t(5.077534951881777, 3.4705937300737975),
                point_t(6.123579367176938, 3.011535973430297),
                point_t(7.19509317490845, 2.6380762575105767),
                point_t(8.313238329988215, 2.3585049171272967),
                point_t(9.486363986576375, 2.161303023096071),
                point_t(10.705327636665864, 2.0212331138280866),
                point_t(11.945153877514414, 1.9102173199682453),
                point_t(13.17315765023744, 1.809186301124977),
                point_t(14.36109285893906, 1.7164714986332732),
                point_t(15.496996837809657, 1.649303225077807),
                point_t(16.591976570897412, 1.6372973860221895),
                point_t(17.67851935298739, 1.709688448779978),
                point_t(18.799664883227504, 1.8804795595980321),
                point_t(19.991652097946794, 2.136739243084669),
                point_t(21.265302890477205, 2.434537521524687),
                point_t(22.592429174929492, 2.704669086062116),
                point_t(23.902474400294807, 2.867121184548795),
                point_t(25.09166806404146, 2.85029070839835),
                point_t(26.043086524088483, 2.609259125097652),
                point_t(26.652447733080454, 2.1375931039250338),
                point_t(26.85242793201401, 1.4691137044686535),
                point_t(26.628494967018153, 0.6692035819260251),
                point_t(26.02168178103253, -0.18155559823398115),
                point_t(25.11760555622142, -1.006379564768898),
                point_t(24.025116405883452, -1.748330782387112),
                point_t(22.8509179397193, -2.379114579545293),
                point_t(21.677410608220484, -2.8989046372494345),
                point_t(20.549633599213525, -3.3284464797196023),
                point_t(19.47404954555798, -3.6970066325006465),
                point_t(18.428118877235995, -4.03065903837725),
                point_t(17.376413821220353, -4.344710279790736),
                point_t(16.28743203457786, -4.642124903560453),
                point_t(15.145701131625193, -4.917408988844138),
                point_t(13.955927068688114, -5.163448575724402),
                point_t(12.73897175917345, -5.377965352027747),
                point_t(11.522248629930768, -5.566770949499337),
                point_t(10.328755973439081, -5.742579274314769),
                point_t(9.168961064014379, -5.9200958304835725),
                point_t(8.03824826001761, -6.109657157896099),
                point_t(6.920315949587658, -6.312250620712428),
                point_t(5.794660618935804, -6.518126523543048),
                point_t(4.644932610021634, -6.709708278195168),
                point_t(3.4649107143799753, -6.867737307373263),
                point_t(2.260028653583159, -6.978267201554641),
                point_t(1.0442557408337068, -7.03776239681459),
                point_t(-0.16606966161878328, -7.0542875588757115),
                point_t(-1.3588072369069344, -7.044293654421869),
                point_t(-2.5303816297154587, -7.02621236869207),
                point_t(-3.6863947381332642, -7.013297528751998),
                point_t(-4.838505912404238, -7.008432460822351),
                point_t(-5.999066982001501, -7.0028603743588915),
                point_t(-7.175790889200835, -6.979296785037214),
                point_t(-8.368585627818142, -6.918229490748481),
                point_t(-9.569711647589543, -6.805030012316254),
                point_t(-10.767046601174657, -6.635224361455823),
                point_t(-11.94902214215861, -6.415981257553375),
                point_t(-13.109196282131988, -6.163280723661876),
                point_t(-14.248637274059103, -5.895784251890166),
                point_t(-15.375189885254338, -5.627562878858319),
                point_t(-16.499885953862307, -5.362158635643097),
                point_t(-17.631774721291695, -5.089888346934321),
                point_t(-18.772915685095313, -4.789109829322025),
                point_t(-19.915071464081414, -4.430836076945749),
                point_t(-21.03890882972334, -3.9850981178714924),
                point_t(-22.11560420349847, -3.4271432752026127),
                point_t(-23.110035081606444, -2.7419476260851483),
                point_t(-23.98447761439906, -1.9263759787162436),
                point_t(-24.701957494081544, -0.9892370255872686),
                point_t(-25.228930757945623, 0.04992862476022748),
                point_t(-25.537495219706553, 1.1624292384916968),
                point_t(-25.607570286934994, 2.3118912431342364)}
        };
    }
    
    std::vector<scalar_t> get_termite_curvature() {
        return {-0.18423682829326793, -0.21920962314687722, -0.20270580394310658, -0.1081419579737491, -0.026430721972191613, -0.009794923015497418, 0.008448495603667725, 0.01874849871548428, 0.01305176932294701, 0.01697512777929988, -0.004247027967801798, -0.008366878076451563, 0.00632696417198358, -0.00632696417198358, -0.012501831457117103, -0.015384615384615385, -0.015384615384615385, -0.012501831457117103, -0.009967755481597177, -0.017038101087054166, -0.005901323946979773, -0.021860608149723352, -0.048024533048442834, -0.05257216305347966, -0.06234646689532352, -0.07071067811865475, -0.08898785700860821, -0.10606215702576095, -0.11278329990586358, -0.11679485026114192, -0.12302980841452124, -0.11089593359626344, -0.09877295966495866, -0.1009421267011747, -0.08328406155298215, -0.0629394031807189, -0.038916303189880985, -0.024040154803290643, -0.017078366911766868, -0.004564304747583513, 0.011921140225759852, 0.011613734102790499, 0.011921140225759852, 0.029387775676131052, 0.02134567165909813, 0.012251277836633432, 0.0035231653050634827, 0.019685037844484873, 0.026295638244101242, 0.02741176256609575, 0.01916017131778767, 0.030280446564178578, 0.04763174030675928, 0.05075678857200672, 0.06589400070487403, 0.07541152224321857, 0.08753762190648166, 0.07685126688541893, 0.06193374927708702, 0.05773168433403844, 0.08476351808151157, 0.10371951569765347, 0.13116589203716028, 0.16427141014301333, 0.12748799414710105, 0.15607653480454875, 0.22360679774997924, 0.22845191651367985, 0.2068415441863557, 0.17388000434264644, 0.16650215022999834, 0.16674999554308648, 0.18239319272941143, 0.015981293741701305, -0.046022687909560274, -0.05633421876407119, -0.05103241247794665, -0.04886137989521312, -0.033398875323323335, -0.02425189460986558, -0.01615681897132433, -0.023799409756413002, -0.010169057808389882, -0.003534384515092582, -0.0011503127946029799, -0.0011503127946029799, -0.0011503127946029799, 0.014737485578028625, 0.006647342620816238, 0.00602850026993932, -0.011815007436031295, 0.0059248769228939305, 0.005598967170859619, 0.012251277836634126, 0.004877964176965177, 0.012251277836634126, 0.012600190862837033, 0.006468697184810692, 0.03632937245845222, 0.049835019513927895, 0.06041093794723016, 0.057972685879756305, 0.06824295211988317, 0.06006205563293226, 0.0854782699091095, 0.08989640454867426, 0.10047207186066513, 0.10075926620309349, 0.09021607777193406, 0.08480345017220095, 0.08159865302645883, 0.0811071103927322, 0.07329729761624655, 0.07420892108611787, 0.052177916801498386, 0.045167492204834854, 0.028664570911483056, 0.030334220470931893, 0.01670797319652672, 0.006687968373875951, 0.01113499986424589, -0.003725914136700408, -0.012501831457117026, -0.011554419162700596, -0.01711775396248015, -0.017687551579034382, -0.005444108891762953, -0.017976659359568894, -0.030273655167383385, -0.017920654343836544, -0.026098089657784023, -0.014516651257954934, -0.015381116596149685, -0.009069412729501106, -0.006529732296425722, -0.018961958442811025, -0.05217106041132056, -0.08019550659365365, -0.1268600191446626, -0.20740623327588736, -0.22188007849009173, -0.23595367508907378, -0.23193473185945085, -0.20918187705430424, -0.21239137180514978, -0.2270825239005704, -0.22621415919185006, -0.22080730297672194, -0.20239401108580507, -0.07518049744881034, 0.02960761897172376, 0.05976368350519382, 0.10322376666441634, 0.09180111254253766, 0.08494732774988116, 0.05860108544215287, 0.035658420864556874, 0.014062805798442365, -0.02248214709838379, -0.04628587458047215, -0.07089952481224693, -0.08943332909338626, -0.11621261648976441, -0.11362376649099415, -0.10168158761870949, -0.07108526439152933, -0.05039035348441037, -0.0386917350563704, -0.018386635683072288, -0.01238316736890605, -0.001600414881312771, 0.010052093916690649, 0.029437689634786195, 0.043689262755276695, 0.042763222137349524, 0.04418666399671974, 0.0355238855997353, 0.03901856411379762, 0.03846254475674226, 0.028123237902280492, 0.028123237902280492, 0.02508477476323802, 0.01855508815711869, -0.010675331023109517, -0.028029919784303015, -0.03685687111378537, -0.0351937467490774, -0.03811198598660611, -0.03757280700502158, -0.027790163443087233, -0.027790163443087233, -0.026110066577000282, -0.021572170925430918, -0.012343879372486809, -0.0011503127946029799, -0.0012349014936631913, -0.01072281054891018, -0.009937957209926653, -0.009937957209926653, -0.009937957209926653, -0.009937957209926653, -0.009937957209926653, -0.009937957209926653, -0.009937957209926653, -0.009937957209926653, -0.008211684794517705, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.00730277901419864, -0.009937957209926653, -0.009937957209926653, -0.009937957209926653, -0.009937957209926653, -0.009937957209926653, -0.009937957209926653, -0.009937957209926653, -0.009937957209926653, -0.009937957209926653, 0.004285537005648987, 0.002794191369843357, -0.0005313931337066062, -0.0011503127946029799, -0.009862101008662104, -0.008557205366924002, -0.0011503127946029799, -0.0011503127946029799, -0.0011503127946029799, -0.0011503127946029799, -0.017076248221334064, -0.006434736798854198, -0.0006212719503550214, 0.0, 0.009184230619574473, 0.007826649037639626, 0.0, 0.011407356307506668, 0.015195839948455383, 0.04331480818242161, 0.11428192604633343, 0.17279700507432294, 0.20570470132667829, 0.22021249682742114, 0.21527086098753626, 0.2086134575785928, 0.20506619579876637, 0.21611797283917394, 0.1510152192663255, 0.16615484683249576, 0.1826284233970983, 0.21329476126827768, 0.1935849592576942, 0.14371262743216406, 0.09719483591434198, 0.05035269576170614, 0.02325168805662591, -0.005244322637304491, -0.009069332152458075, -0.00536425961883157, -0.009082098501643848, -0.01636563055592553, -0.012590464725697803, -0.015041420939904671, -0.05783059399348131, -0.09864990796177828, -0.1290000590454386, -0.10596251828199206, -0.10012009785313015, -0.09154500413914904, -0.10991062628355126, -0.0830888598918393, -0.04430893391742571, -0.02175713172881657, 0.05234653486920753, 0.11009007473694606, 0.1433046089223229, 0.14609325712188978, 0.15057988310128945, 0.13915007534941579, 0.16153667019564344, 0.14592321716377402, 0.10666897126810139, 0.08389279492974541, 0.0482981788011037, 0.032249894025200815, 0.020513423654017184, 0.01812563293690196, 0.0013507433022257856, -0.004716107707691028, -0.02395596150902906, -0.03318997347481728, -0.03587861668452074, -0.045203975942389445, -0.031475594237181756, -0.04840640142764807, -0.05630602208806976, -0.08922226002320173, -0.09647364962346196, -0.1253400464143864, -0.13232301396587595, -0.13625097309076883, -0.1552935491158776, -0.15524977218645777, -0.1579171763763907, -0.17256414716288035, -0.1813906389106644, -0.17823937575773965, -0.17811673310405182, -0.15878316700086173, -0.13918454046250361, -0.14441565888712773, -0.12533981197686223, -0.12563698669604667, -0.10268664845416539, -0.09009907753694485, -0.07832961378876294, -0.053656330653890166, -0.040991888526823325, -0.04968610542546462, -0.04985008206542085, -0.0284459345715423, -0.011451052101058892, 0.009059814658382079, 0.007984689280197018, -0.010317258698775339, 3.1776437161564794e-16, 0.016230421442561576, 0.033341721683855345, 0.07661738953012617, 0.1326496777233487, 0.16721386830300403, 0.20035877104620253, 0.19868075876282587, 0.22650303446479136, 0.22242200537027332, 0.20733757793551397, 0.19247301798398897, 0.2, 0.1993630557072251, 0.1920753384895164, 0.17726162559245368, 0.1414213562373095, 0.08381163549234938, 0.060661596556698213, 0.03559910975091625, 0.0009250681680205791, -0.02808020134863881, -0.08381163549234938, -0.13453455879926252, -0.19156525704423025, -0.1867401766716897, -0.1486588292494331, -0.13793103448275862, -0.15238786355189188, -0.2302621748542738, -0.24159791990365867, -0.1734462744870962, -0.19154085227222933, -0.20601048104984193, -0.21081851067789198, -0.1469565762899582, -0.07647380148415728, -0.04811423262410498, -0.03104326028222251, -0.008409322096675286, 0.012093704037586564, 0.029553891462911736, 0.0396412642859009, 0.07748388442260501, 0.11433239009500462, 0.17240958148343752, 0.21099559714964466, 0.20055136532443846, 0.21367958934185355, 0.19815424292199538, 0.2010953240452679, 0.23242986940942034, 0.1799550129659386, 0.17252930253728663, 0.19754591932991616, 0.2287596317043831, 0.18018443378954707, 0.14099124284695108, 0.10964800633977935, 0.06558133542592917, 0.019992439799113443, 0.005168903695170193, -0.004382029120956859, -0.009940808847613962, -0.0097375795390451, 0.016101889584332663, 0.02871598031534612, 0.01353315958553869, 0.02243882694011406, 0.03549030646660935, 0.04982514032316813, 0.04850712500726626, 0.04850712500726626, 0.03979177295545294, 0.025650072159732407, -0.015027958723152856, -0.06148418632608446, -0.0673572268758584, -0.11696203413124805, -0.16056516834828036, -0.17510890889236547, -0.1911072842510348, -0.19210351249785868, -0.21247293965601285, -0.20397043336230386, -0.1956774564406011, -0.17884967184189282, -0.16774533580880416, -0.13717471583476262, -0.09102181613587694, -0.0587187459771321, -0.023275069584991037, -0.011288610138958917, -0.008291923863777112, 0.0016799534246758022, 0.0005000926380435263, 0.0013838982042435478, -0.01700696585075433, -0.035270261268818374, -0.05255151490433673, -0.07223767930377857, -0.08215782731965669, -0.07609151250084596, -0.08353530327563537, -0.09014153769324836, -0.08467002969579031, -0.0817907037498859, -0.06157868979300467, -0.0490415151094266, -0.01968787915007926, 0.013848652487385697, 0.04333286363359272, 0.05619110702378477, 0.12594529853040626, 0.20814794182907934, 0.21049392463368702, 0.21304820777621813, 0.21571478541257988, 0.22406337051031458, 0.20489021439895616, 0.2213949439727063, 0.27019904016407725, 0.3022130242434419, 0.3041131995673229, 0.148127816155637, -0.14575973943220488, -0.2014826248668375, -0.2180051012594344, -0.21239368541091194, -0.2058945713429762, -0.1972316771231892, -0.19413987809924046, -0.20733757793551402, -0.2114027653249252, -0.17925163190605434, -0.12649110640673517, -0.09550763994093645, -0.06514668192807298, -0.024951222434391348, -0.0050689474339545276, 0.0, -0.014085451877451978, -0.007066731557619746, 0.01661022489021771, 0.04799470529280958, 0.06693027344765601, 0.11954219518488808, 0.18116970035915175, 0.22838139978040622, 0.173047551616033, 0.19756156606142625, 0.21165364767558348, 0.21942158412190516, 0.19837931759725558, 0.20146384988947855, 0.1910381616912319, 0.203749077551571, 0.20247108965666105, 0.17962162476758972, 0.0843076535515669, 0.06987733170031961, 0.04968629893266639, 0.022472273365990506, 0.02166461689970939, 0.008924979107485291, 0.0141853069678989, 0.012501831457117103, 0.001669188727896911, -0.010260811374030948, -0.002930961732747527, -0.015461168437893032, -0.028013309483758273, -0.024540478685347582, -0.045873734290643536, -0.057078179298539436, -0.06603778502241613, -0.07932896763298987, -0.07581385951237801, -0.07143145241759508, -0.07421120161452928, -0.06891037625986424, -0.05470793627265788, -0.055513837546373755, -0.05312387464087993, -0.037173802466105244, -0.020402783665320504, -0.006516752964893083, 0.0004458547756811735, -0.008052675732157259, -0.003771529215811416, 0.01091190046115695, 0.0025961303498821873, 0.006991292265106118, 0.022072753697368046, 0.016541899681885967, 0.006550501518993846, 0.011195356565038646, 0.008973033064203547, 0.0199784119622835, 0.02114512438833464, 0.006845658710200208, 0.010877850680806976, 0.01742102097005372, 0.006929582124702697, 0.015659960483992572, 0.022723754657314996, 0.01709395030158208, 0.01607055340591772, 0.015890995714921606, 0.010531930186480779, 0.01322302760252788, 0.023256878083775086, 0.023579473464865355, 0.028808195766917018, 0.022995342477611005, 0.024589735479677816, 0.028342785893429992, 0.03242346456490011, 0.030098290950300314, 0.04126616786465368, 0.04618077567902838, 0.04624813631714583, 0.03257660811403403, 0.034665487866675386, 0.03586452084668838, 0.02411333833903771, 0.027758383107172078, 0.016147807443072867, 0.013069913409450918, -0.005716015125163543, -0.031213163876493283, -0.05501245242726595, -0.04677447494869335, -0.05346435324150066, -0.056702348854802284, -0.05257205330613895, -0.06385656188718705, -0.054366529784810616, -0.051054449667757885, -0.02955629163395946, -0.010234765854455704, 0.00195406812599771, -0.005520129844101099, -0.024286817018911715, -0.03776665747946011, -0.03941576404290835, -0.05652727951071563, -0.0695049948131819, -0.0712334472201724, -0.09139378126688902, -0.09139378126688902, -0.0835260190889087, -0.08197974428695133, -0.04910988928684138, -0.03157359080773931, -0.025613331027463437, 0.01691000505020437, 0.02991118125397361, 0.01077703453343649, 0.010193271875321767, -0.0038741942211305263, -0.039411519255945264, -0.0650954527553367, -0.1253513905284893, -0.14976170241757603, -0.16359133261652048, -0.17050168975576457, -0.1766688544322517, -0.17541160386140575, -0.17540936956150124, -0.17141490688178543, -0.1537953940668294, -0.13497403469521901, -0.09668435585605445, -0.0789369157395428, -0.045991428921984025, -0.02468383475881755, -0.010459598726238096, -0.010459598726238096, -0.008160986383105428, 0.004221782878802376, 0.006495075932574156, 0.015553457419976232, 0.04508464830331752, 0.05815762865650668, 0.07550546472129996, 0.0923408432429546, 0.10739757088859207, 0.11940446282519869, 0.12332037708226351, 0.1304631693887242, 0.14030681097778888, 0.13864732236422642, 0.13258244460315338, 0.12615196595519906, 0.11462309472758914, 0.09877295966495897, 0.08192493645463088, 0.07329378118237488, 0.06980870001410194, 0.062125912280989806, 0.04263304482227313, 0.03978548670659206, 0.023955961509029327, 0.015466378406695696, 0.01181500743603162, 0.01181500743603162, 0.007890873030817095, -0.009853946286416476, -0.016633372160305866, -0.008567071501722404, -0.014274794957135396, -0.027394445947513453, -0.0201692472192513, -0.01211702293127827, -0.01181500743603162, -0.02052349468016659, -0.02240218022564736, -0.013235861977366856, -0.015684476968449466, -0.04069286518799769, -0.026746740510925734, -0.02460724773328534, -0.026870847585047597, -0.03432075670589594, -0.035878616684521025, -0.028169042964294733, -0.023955961509029327, -0.023955961509029327, -0.021710547899485844, 0.005924876922893938, 0.0030605175592008363, 0.015237541819069835, 0.012513108269093501, -0.005792922183917525, 0.0, 0.011744122084970814, 0.024631714004245353, 0.02783269390582717, 0.03759621465078998, 0.030077594795533142, 0.027394690053949688, 0.020654139776023765, 0.030591557956420306, 0.06580516474949732, 0.051882510681350295, 0.03356285683222314, 0.013722308608979758, 0.009994871291996629, -0.0007325434073682016, -0.0016004148813128245, -0.010717863233603762, -0.01062424912316512, -0.011323886713434733, -0.037504289096085226, -0.02009366613368944, -0.01968625526395581, -0.013331537399957264, -0.014927117207206738, -0.02270761766412059, -0.012251277836633946, 0.004509363579975164, 0.005188011982334918, -0.007772243567182749, 1.8576280673466647e-16, -0.00917787857770254, 0.00467728752017462, 0.010459598726238258, 0.012976200875975464, 0.018625332331436675, 1.662088270783858e-16, -0.007772765432423825, -0.008429794925277663, -0.0011505965383248082, 0.0, 0.0, -0.006676303819569609, -0.009937957209926653, -0.009937957209926653, -0.008211684794517723, 0.0019409023876461682, 0.009089778573050418, 0.01072281054891018, 0.01160830430091997, 0.01160830430091997, 0.01160830430091997, 0.013446594391428039, 0.02101901566232991, 0.022709318232146385, 0.01160830430091997, -0.0008385110714398533, -0.019286127327446567, -0.024463001648824527, -0.024390243902439022, -0.024390243902439022, -0.024390243902439022, -0.021032027319630792, -0.029213573411978326, -0.019780084942997947, 0.025491966026894502, 0.07964308870306053, 0.16544413591339566, 0.20171183286870195, 0.21906957714229094, 0.21730051126290956, 0.21385668504178487, 0.214788440783006, 0.18890777277612922, 0.15610421249248385, 0.18786953849907945, 0.1861251675925329, 0.20599747821074355, 0.19275709463637405, 0.15721548548591527, 0.11730290113343818, 0.08306226406515256, 0.04985454291328648, 0.0318525168781939, 0.02856527539646788, 0.0415814739331676, 0.04254356298115171, 0.04468994796958379, 0.04254356298115171, 0.031210949694043637, 0.012687685104004708, 0.011004220239952565, 0.003305783598633968, -0.004689316173760605, -0.021917808219178086, -0.026247176035170634, -0.02387746560182273, -0.023316085404864226, -0.023955961509029317, -0.022402180225647335, -0.01255566989237646, -0.011300776509212157, 0.004645651815829172, 0.013890371565508694, 0.02191780821917807, 0.01769775682228964, -0.006192809144302391, -0.007492706301538812, 0.0, 0.0, 0.0, 0.0, -0.007772765432423982, -0.011608304300919954, -0.011004220239952551, -0.009622753968534382, 0.007232901208411068, 0.004243195473688628, -0.009089778573050418, -0.009016095293689307, -1.534235326877411e-16, -1.662088270783862e-16, -0.00857159972667114, -0.011608304300920132, -0.01192114022576035, -0.011608304300920132, -0.016868962310738343, -0.02577039190119676, -0.03108145426821969, -0.034124850346967044, -0.0538916079354227, -0.06006205563293276, -0.04508464830331727, -0.042952462972764165, -0.047392675729258565, -0.05751934883932676, -0.03540526985746395, -0.010446625402202894, 0.010323508139053693, 0.019312781903878543, 0.03052596199542587, 0.04413410521659529, 0.03952952106041399, 0.041807654229720465, 0.045968649645008644, 0.051836677892909785, 0.03179608161615264, 0.013388853404658202, -0.005394611219921502, -0.020834544707856277, -0.026906911759852496, -0.044265982636007346, -0.05711924950177663, -0.06715206148556327, -0.06740769359590609, -0.0686686109604242, -0.06501109926847291, -0.06859943405700354, -0.06679698809705838, -0.054366529784810616, -0.058948194330433384, -0.058948194330433384, -0.06026114842284758, -0.07407585328409556, -0.09228391066892794, -0.11449899792055052, -0.1342534871791348, -0.14637799137921104, -0.16546941375449625, -0.17267083742860226, -0.17192101931786952, -0.17013004222631367, -0.16360996965814906, -0.15594294272028, -0.13762928110631673, -0.12789300580619992, -0.10818195943188613, -0.09065645491878305, -0.06386184666360642, -0.02791259528622889, 0.0013318833870896313, 0.024060548554504634, 0.01961965565420802, 0.018006829787633932, 0.005003174267819483, 0.01145973859415527, 0.017006965850753987, 0.023716196953357797, 0.029942277463196356, 0.03209859187344426, 0.018401726672713545, 0.001170087578844979, -0.011988466648607678, -0.006333958606037064, 0.00367909741252467, 0.012326167671437035, 0.013512199428344905, 0.0025961303498824996, -0.007550967394228455, -0.01994047824771476, -0.020835010590499765, -0.004833611861458166, 0.007447913117766815, 0.015731554261131396, 0.021333846702784594, 0.016409298561554667, 0.012973899674063488, 0.022718269802068806, 0.026616911505783273, 0.030393910814726866, 0.03865439588482638, 0.030646880832318812, 0.04544027190139353, 0.03582574822588689, 0.02805845695998311, 0.023997239479122856, 0.01599616138184724, 0.012227785431821908, 0.01922751790833498, 0.02972957113224371, 0.02330202806127006, 0.022301402120966353, -0.004562002075011932, -0.004064566846662673, -0.003647523999320366, -0.0011453417757242893, 0.019069442593494648, 0.02252386506423065, 0.018482552801663336, 0.013515148135749425, 0.027022640946662697, 0.04025996938594176, 0.06279423777304223, 0.07178371734886159, 0.08148143396632211, 0.08577851879397919, 0.0683814494468507, 0.06260708611450579, 0.06173868345734082, 0.05681766593827481, 0.04726876331246494, 0.04405319108527728, 0.04148031886063087, 0.03467595421997113, 0.03465834966066909, 0.03708086485884678, 0.04771722366696871, 0.05824076263283536, 0.06399276707762344, 0.07692307692307693, 0.08045619304950723, 0.07214320812669911, 0.062017367294604234, 0.052257321713990446, 0.04110534215196261, 0.03391024856670084, 0.04670278914899727, 0.044570700592636826, 0.05565197790690477, 0.041154670794599814, 0.04641526374150367, 0.05149811491420649, 0.06381534447172757, 0.08289657185507611, 0.08495569974376767, 0.09693875423128932, 0.07559311217492656, 0.06607284432173974, 0.03171933755705161, 0.029289893401422764, 0.019555253923726017, 0.03472566708684485, 0.028338165008783597, 0.010162297262080048, 0.01474926253687315, -0.014749262536873238, -4.386066270124075e-17, -0.0027281936639936585, 0.022026993486616485, 0.03020322103657247, 0.03059155795642018, 0.015553457419976274, 0.012251277836633778, 0.012251277836633778, 0.004877964176964742, 0.02664089555038549, 0.012251277836633778, 0.024034819123427965, 0.01929422361115055, 0.011290981859916975, 0.022758388132951453, 0.02812323790228075, 0.03435456710879308, 0.042952462972764255, 0.047392675729258676, 0.03970202193957426, 0.047392675729258676, 0.03668085705727225, 0.032037277110876464, 0.032269185707102854, -0.04306505200950911, -0.10846590914873362, -0.16811849443775093, -0.21045345670586257, -0.24241835497615005, -0.24794050141863289, -0.24753126796972869, -0.22992385033477664, -0.2245682263216611, -0.20739355722665445, -0.21475019687726363, -0.22930145165427537, -0.22231001578113407, -0.16806800705618208, -0.13199183914407556, -0.08862280499284834, -0.05178965635138488, -0.0441233669792537, -0.024809809753371076, -0.019084351745913327, -0.02387879807935663, -0.01217500709225339, 0.009878442896204022, 0.00879685210462999, -0.004330496006434767, 0.008280645365064265, 0.008095882037381277, 0.02483268570271754, 0.019884428945987798, 0.01770022804140085, 0.03272744668628568, 0.016735083333470232, 5.500250913503217e-17, -0.006942033117194518, 0.0001377945670329057, 2.88619937258047e-17, 0.0, -0.019200755211935306, -0.03092043548818523, -0.03693951103987006, -0.05405097644545173, -0.04750695953704497, -0.03922156165514439, -0.033375577856687495, -0.0315588749670884, -0.03617293075045202, -0.04388498383890967, -0.022634859431558796, -0.013532239201685399, -0.01313433523704578, -4.2853681455010724e-17, 0.002504515132149642, 0.01402936514724667, 0.011743377928608289, 0.022283611571558944, 0.03451826651163384, 0.07917265864506466, 0.09370425713316366, 0.159263655649664, 0.1921785392871812, 0.21261752800686595, 0.2079163283748721, 0.21707439945428286, 0.22079229957621158, 0.22111155303104224, 0.19777464607746503, 0.20318368223852043, 0.2112361719027836, 0.22614038395275382, 0.19567435576408201, 0.15221810837876046, 0.12038338358063798, 0.06700176264681326, 0.03392621711151629, 0.026879349054558386, 0.020251665692567065, -0.0028281177736668574, 0.008366878076451529, 0.004949022854124339, -0.004833611861458364, -0.018996982387406176, -0.00612781647116908, 0.0032055249757534767, 0.013512199428344824, 0.0010408864421786602, 0.005666571205334094, 0.018612365691906926, 0.017979707305277103, 0.02255728313569821, 0.019926719432894348, 0.03114881369631374, 0.024670829220278565, 0.022837681643611208, 0.013335206804464055, 0.027500803475486214, 0.020689077780819205, 0.027433102087779184, 0.019177829856066652, 0.011416961946985769, 0.013293362351117421, 0.005632657629989713, 0.0036000096975391848, 0.005779194832101639, 0.0172711829269031, 0.008406643906899595, 0.006349907050851363, -0.010779228036657961, -0.003056575586992213, 0.00836809096432098, 0.011698664924617607, 0.01663780661615406, 0.01663780661615406, 0.01456812661951192, 0.00386224986082048, 0.0016114044865936726, 0.008440628117942444, 0.016637806616154088, 0.013032503003369706, -0.006453262780738404, -0.0030713172627474055, -0.008028956459036974, -0.010900950611793928, -0.005828337014331812, -2.4454689265941907e-17, 0.004735598687279406, 0.004239735926171817, 0.003600009697539157, 0.004963910767281998, 0.014869131241231352, 0.012048925582277306, 0.017831573790537274, 0.034166847713556076, 0.027050133794768373, 0.02433848682012207, 0.0275219726502881, 0.022316045891167553, 0.020383439144668704, 0.016971246364048977, 0.018992566024052724, 0.008789829563824977, -0.0005989039312290736, -0.027794323278914287, -0.030814848630853816, -0.032918881467812695, -0.04401548339229668, -0.04170118646884824, -0.04176822910040251, -0.037829552906168115, -0.04011365401324665, -0.028772837920195332, -0.017271182926903036, -0.012917941467210998, -0.012227785431821908, -0.014438995015168314, -0.017861018533618243, -0.0328806900628519, -0.041105342151962654, -0.052613364176465637, -0.06316182261095987, -0.08838112650051666, -0.1060493468275142, -0.10820980018662811, -0.10883207553814433, -0.11666081648888318, -0.11767028793513658, -0.11392052571985642, -0.11168288112978011, -0.10714651076428737, -0.09544653439516147, -0.07734040777231227, -0.06105869231199675, -0.05722766237963045, -0.05077957928975495, -0.03611577384770204, -0.02234060799029685, -0.021224876985497146, -0.0064104375097813115, 0.005018500615079715, -0.006453262780738157, -0.005701282714722816, -0.00422881447961006, 0.00246602756247424, -0.002988832647498241, -0.006256558196895168, -0.006367080850585438, 0.0009120530607773445, -0.005965622597058988, -0.010779228036657946, -0.011454027136047203, -0.01958353227729641, -0.02802448300221586, -0.03407005241568884, -0.03008386284270548, -0.05225171642937135, -0.07653206827803614, -0.08296040813645782, -0.08321432615569324, -0.08717466390698286, -0.07864935229881446, -0.07576803602043596, -0.06918942091361396, -0.0723098030411675, -0.07023347541482226, -0.042159171364551495, -0.008093613971093608, -3.32417654156772e-16, 0.0025528057570801885, 0.011921140225759862, 0.011613734102790457, 0.003415439756968106, 0.010345008498983267, 0.0221055379394909, 0.020530375347451817, 0.008433436720412215, -0.009288664284874616, -0.011004220239952251, -0.010459598726237763, -0.011004220239952251, -0.008355057847226295, 0.015537090433899082, 0.013621434837970604, 0.018564561014911322, 0.018844406755669067, 0.023365270717331535, 0.04804896228314452, 0.07653527372568608, 0.09819329453096984, 0.15173070188708665, 0.1953959233354394, 0.11964409535601889, 0.14760153870221876, 0.1207406700501949, 0.16254851075486293, 0.20779321396410372, 0.2270383045932499, 0.21510973058909238, 0.20614171037128637, 0.20601048104984193, 0.2047826123561298, 0.048459269826666526, 0.04312562173865923, 0.008391071872449918, 0.00850323709435147, 0.004763583415353414, -0.01371974183145645, -0.027428580851745963, -0.027390743546597353, -0.025293163154477917, -0.021710547899485698, 0.005924876922893946, 0.003060517559201332, 0.001532366095462045, -0.0005313931337063043, -0.0011503127946026535, 0.006647342620816578, 0.007862851890431222, 0.0123438793724865, 0.013984246113709596, 0.0123438793724865, 0.004034284340349938, 0.0011503127946026535, 0.0011503127946026535, -3.097593973685978e-16, 0.0033582347871355923, 0.01839854716194003, 0.022351280636053267, 0.021294358797497807, 0.028640512155983462, 0.03345697985290845, 0.04282192917579905, 0.05147290265511386, 0.05653569066951057, 0.06861436933264432, 0.07040333485408434, 0.0564398197129954, 0.06503004015023718, 0.07079944574032826, 0.06181147237331544, 0.05907934207608588, 0.0484658041418883, 0.04325503395105096, 0.0420381764219817, 0.03242346456490075, 0.022524968538771624, 0.022106794582934578, 0.0039496662518745335, -0.006499272899373568, -0.0075386645913458426, -0.01963567010007513, -0.023896619349605856, -0.052250126544722955, -0.12701073389395434, -0.1940285000290662, -0.2291610632697769, -0.19282924291540773, -0.2047769643422278, -0.2078218153242193, -0.21511475294584728, -0.23493184781303858, -0.20363326478252114};
    }
}
